SystemOrganization addCategory: #'Arduino-Core-AFMotorShield'!
SystemOrganization addCategory: #'Arduino-Core'!
SystemOrganization addCategory: #'Arduino-Core-Devices'!
SystemOrganization addCategory: #'Arduino-Core-Firmata'!
SystemOrganization addCategory: #'Arduino-Core-Firmware'!
SystemOrganization addCategory: #'Arduino-Core-PinMode'!

ArduinoPinMorph subclass: #AFMotorPinMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-AFMotorShield'!

!AFMotorPinMorph class methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:15'!
devices
	^ {AFMotorMorph}! !

!AFMotorPinMorph methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:47'!
move: aDevice
	self center x > owner center x
		ifTrue: [aDevice left: self right + 50]
		ifFalse: [aDevice right: self left - 50].
	aDevice 
		bottom: self top - 50.
	self controlDeviceBounds: aDevice! !

Object subclass: #Arduino
	instanceVariableNames: 'protocol digitalPins analogPins keepAliveProcess'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core'!

!Arduino commentStamp: '<historical>' prior: 0!
I represent the Arduino board. I have a collection of pins and a protocol (i.e. Firmata). 
Below you will find a code example using the Firmata protocol:

---------------------------------------------------------------------------------------
| arduino digitalPin analogPin led button ldr servo |

"Connection"
arduino := Arduino on: Firmata new.
arduino connectOnPort: 'COM5'.
arduino isConnected.
arduino disconnect. "Don't forget to disconnect me after finish using me"

"Digital pins - output"
digitalPin := arduino digitalPin: 4.
digitalPin setOutput.
digitalPin value: 1.

"Digital pins - input"
arduino activateDigitalReports.
digitalPin setInput.
digitalPin value.

"Analog pins (analog pins are input only)"
analogPin := arduino analogPin: 0.
analogPin activate.
analogPin value.

"To avoid configuring pins, you can also attach/detach devices:"

"Led (digital output)"
led := LightEmittingDiode new.
led attach: (arduino digitalPin: 13).
led value: 1.
led value: 0.

"Button (digital input)"
button := EButton new.
button attach: (arduino digitalPin: 2).
button value.

"Ldr (analog input)"
ldr := Photoresistor new.
ldr attach: (arduino analogPin: 5).
ldr value.

"Servo (currently servos can only be attached to pins 9-10)"
servo := Servo new.
servo attach: (arduino digitalPin: 9).
servo angle: 0.
servo angle: 90.
servo angle: 180.

"To see the current list of available devices evaluate:"
AttachableDevice allSubclasses!

Arduino subclass: #AFMotorShield
	instanceVariableNames: 'motorPins'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-AFMotorShield'!

!AFMotorShield class methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:53'!
defaultProtocol
	^Firmata! !

!AFMotorShield methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 17:28'!
initialize
	super initialize.
	motorPins := Dictionary
		newFrom: ((1 to: 4) collect: [:i | 
			i -> (AFMotorPin on: self number: i)])! !

!AFMotorShield methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:18'!
motorPin: t1 
	^ motorPins
		at: t1
		ifAbsent: [self inaccessiblePinError]
! !

!AFMotorShield methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 17:30'!
reconfigurePins
	digitalPins values , analogPins values, motorPins values
		do: [:each | each refresh]! !

!Arduino class methodsFor: 'examples' stamp: 'Richo 2/22/2012 13:01'!
buttonExample
	"
	This example requires a button attached to the digital pin 9.
	It creates a TextMorph that shows if the button is pressed..
	
	self buttonExample.
	"
	
	| arduino port button textMorph process buttonMorph |
	"Just to be sure, disconnect all other instances"
	Arduino instancesConnected do: [:each | each disconnect].
	
	"Create and connect your arduino. Install the firware automatically if not installed already"
	arduino := Arduino new.
	port := (UIManager default request: 'Arduino port:' initialAnswer: 'COM8').
	[arduino connectOnPort: port]
		on: FirmwareNotInstalled
		do: [:ign |
			arduino disconnect.
			ArduinoDirectMode
				uploadOnPort: port
				arduinoType: (ArduinoType
					withFriendlyName: (UIManager default
						chooseFrom: ArduinoType allFriendlyNames
						values: ArduinoType allFriendlyNames
						title: 'Arduino Type:')).
			arduino connectOnPort: port].

	"Create a button and attach it to pin 9"
	button := EButton new.
	button attach: (arduino digitalPin: 9).
	
	"Create an arrow to show potentiometer value"
	textMorph := TextMorph new contents: 'Not pressed'.
	
	"Create and start a process to update the arrow's rotation"
	process := [[textMorph contents: (button value = 1 ifTrue: ['Pressed'] ifFalse: ['Not pressed']).
				textMorph center: World center.
				10 milliSeconds asDelay wait] repeat]
					forkNamed: 'buttonExample process'.
	
	"Create clean up button"
	buttonMorph := SimpleButtonMorph newWithLabel: 'Clean up'.
	buttonMorph
		actionSelector: #value;
		target: [process terminate.
			arduino disconnect.
			textMorph delete.
			buttonMorph delete].

	"Open morphs in World"
	textMorph openInWorld.
	buttonMorph openInWorld.
	textMorph center: World center.
	buttonMorph center: World center.
	buttonMorph top: textMorph bottom + 10.
	! !

!Arduino class methodsFor: 'instance creation' stamp: 'Richo 12/23/2011 14:57'!
defaultProtocol
	^ ArduinoDirectMode ! !

!Arduino class methodsFor: 'just for testing' stamp: 'Richo 1/31/2011 09:24'!
instancesConnected
"Just for testing. Do not use!!

Update 31/1/2011: I read this comment after a while and I thought the 'Do not use!!' warning was funny and a little exagerated :). This method was certainly used for testing while developing one of the earlier versions but it does no harm if you execute it."
^self allInstances select: [:each| each isConnected]! !

!Arduino class methodsFor: 'examples' stamp: 'Richo 2/22/2012 13:01'!
ledExample
	"
	This example opens an EllipseMorph in the center of the screen that
	lets you turn on/off the a led connected to digital pin 13.
	
	self ledExample.
	"

	| arduino port led ledMorph button |
	"Just to be sure, disconnect all other instances"
	Arduino instancesConnected do: [:each | each disconnect].
	
	"Create and connect your arduino. Install the firware automatically if not installed already"
	arduino := Arduino new.
	port := (UIManager default request: 'Arduino port:' initialAnswer: 'COM8').
	[arduino connectOnPort: port]
		on: FirmwareNotInstalled
		do: [:ign |
			arduino disconnect.
			ArduinoDirectMode
				uploadOnPort: port
				arduinoType: (ArduinoType
					withFriendlyName: (UIManager default
						chooseFrom: ArduinoType allFriendlyNames
						values: ArduinoType allFriendlyNames
						title: 'Arduino Type:')).
			arduino connectOnPort: port].

	"Create a led and attach it to pin 13"
	led := LightEmittingDiode new.
	led attach: (arduino digitalPin: 13).

	"Create an EllipseMorph to turn on/off the led"
	ledMorph := EllipseMorph new.
	ledMorph
		on: #mouseUp
		send: #value
		to: [	led value: (led value bitXor: 1).
			ledMorph color: (Color h: led value * 127 s: 1 v: 1)].

	"Create clean up button"
	button := SimpleButtonMorph newWithLabel: 'Clean up'.
	button
		actionSelector: #value;
		target: [arduino disconnect.
			ledMorph delete.
			button delete].

	"Open morphs in World"
	ledMorph openInWorld.
	button openInWorld.
	ledMorph center: World center.
	button center: World center.
	button top: ledMorph bottom + 10.! !

!Arduino class methodsFor: 'instance creation' stamp: 'Richo 3/30/2012 16:29'!
new
	^ self on: self defaultProtocol new! !

!Arduino class methodsFor: 'instance creation' stamp: 'Richo 3/30/2012 16:29'!
on: aProtocol
	^ self basicNew initializeWith: aProtocol! !

!Arduino class methodsFor: 'examples' stamp: 'Richo 2/22/2012 13:01'!
potentiometerExample
	"
	This example requires a potentiometer attached to the analog pin 1.
	It creates an arrow that shows the value of the potentiometer by 
	constantly updating its heading.
	
	self potentiometerExample.
	"
	
	| arduino port potentiometer arrowMorph process button |
	"Just to be sure, disconnect all other instances"
	Arduino instancesConnected do: [:each | each disconnect].
	
	"Create and connect your arduino. Install the firware automatically if not installed already"
	arduino := Arduino new.
	port := (UIManager default request: 'Arduino port:' initialAnswer: 'COM8').
	[arduino connectOnPort: port]
		on: FirmwareNotInstalled
		do: [:ign |
			arduino disconnect.
			ArduinoDirectMode
				uploadOnPort: port
				arduinoType: (ArduinoType
					withFriendlyName: (UIManager default
						chooseFrom: ArduinoType allFriendlyNames
						values: ArduinoType allFriendlyNames
						title: 'Arduino Type:')).
			arduino connectOnPort: port].

	"Create a potentiometer and attach it to pin 1"
	potentiometer := Potentiometer new.
	potentiometer attach: (arduino analogPin: 1).
	
	"Create an arrow to show potentiometer value"
	arrowMorph := PolygonMorph arrowPrototype.
	arrowMorph setVertices: {
		arrowMorph vertices first.
		arrowMorph vertices first + (0@ -100)}.
	
	"Create and start a process to update the arrow's rotation"
	process := [[arrowMorph rotationDegrees: (0.264 * potentiometer value - 135).
				10 milliSeconds asDelay wait] repeat]
					forkNamed: 'potentiometerExample process'.
	
	"Create clean up button"
	button := SimpleButtonMorph newWithLabel: 'Clean up'.
	button
		actionSelector: #value;
		target: [process terminate.
			arduino disconnect.
			arrowMorph delete.
			button delete].

	"Open morphs in World"
	arrowMorph openInWorld.
	button openInWorld.
	arrowMorph center: World center.
	button center: World center.
	button top: arrowMorph bottom + 10.
	! !

!Arduino methodsFor: 'connection' stamp: 'Richo 9/11/2009 17:21'!
activateDigitalReports
protocol activateDigitalPort: 0; activateDigitalPort: 1.
! !

!Arduino methodsFor: 'connection' stamp: 'Richo 9/11/2009 17:24'!
activateReports
self activateDigitalReports.
analogPins do: [:each | each activate]! !

!Arduino methodsFor: 'accessing' stamp: 'Richo 8/31/2009 16:07'!
analogPin: aSmallInteger 
	^ analogPins at: aSmallInteger ifAbsent: [self inaccessiblePinError ]! !

!Arduino methodsFor: 'accessing' stamp: 'Richo 8/15/2011 10:11'!
analogPins
	^ analogPins values! !

!Arduino methodsFor: 'connection' stamp: 'Richo 11/23/2012 12:52'!
connectOnPort: portName
	self connectOnPort: portName boardType: self typeClass default! !

!Arduino methodsFor: 'connection' stamp: 'Richo 12/11/2012 13:29'!
connectOnPort: portName boardType: anArduinoType
	"I reconfigure each pin every time I connect to a board"
	protocol connectOnPort: portName boardType: anArduinoType.
	500 milliSeconds asDelay wait.
	self reconfigurePins;
		startKeepAliveProcess! !

!Arduino methodsFor: 'connection' stamp: 'Richo 9/11/2009 17:21'!
deactivateDigitalReports
protocol deactivateDigitalPort: 0;deactivateDigitalPort: 1.
! !

!Arduino methodsFor: 'connection' stamp: 'Richo 1/12/2010 12:59'!
deactivateReports
self deactivateDigitalReports.
analogPins do: [:each | each deactivate]! !

!Arduino methodsFor: 'accessing' stamp: 'Richo 8/31/2009 16:06'!
digitalPin: aSmallInteger 
	^ digitalPins at: aSmallInteger ifAbsent: [self inaccessiblePinError]! !

!Arduino methodsFor: 'accessing' stamp: 'Richo 8/15/2011 10:11'!
digitalPins
	^ digitalPins values! !

!Arduino methodsFor: 'connection' stamp: 'Richo 12/11/2012 13:30'!
disconnect
	protocol disconnect.
	self stopKeepAliveProcess! !

!Arduino methodsFor: 'errors' stamp: 'Richo 4/6/2011 10:37'!
inaccessiblePinError
self error: 'Inaccessible pin' translated! !

!Arduino methodsFor: 'initialize-release' stamp: 'Richo 10/24/2011 14:06'!
initialize
	digitalPins := Dictionary newFrom: ((2 to: 13)
				collect: [:i | i-> (ArduinoDigitalPin on: self number: i)]).
	analogPins :=Dictionary newFrom: ( (0 to: 5)
				collect: [:i | i -> (ArduinoAnalogPin on: self number: i)])! !

!Arduino methodsFor: 'initialize-release' stamp: 'Richo 12/23/2011 16:08'!
initializeWith: aProtocol
	protocol := aProtocol.
	self initialize.! !

!Arduino methodsFor: 'connection' stamp: 'Richo 8/24/2009 16:14'!
isConnected
^protocol isConnected! !

!Arduino methodsFor: 'keeping alive' stamp: 'Richo 12/11/2012 13:27'!
keepAliveTime
	^ protocol keepAliveTime! !

!Arduino methodsFor: 'accessing' stamp: 'Richo 8/15/2011 10:10'!
pins
	^ self digitalPins, self analogPins! !

!Arduino methodsFor: 'accessing' stamp: 'Richo 10/24/2011 14:00'!
protocol
	^ protocol! !

!Arduino methodsFor: 'accessing' stamp: 'Richo 12/23/2011 15:35'!
protocol: aProtocol
	self disconnect.
	protocol := aProtocol ! !

!Arduino methodsFor: 'connection' stamp: 'Richo 9/8/2009 10:33'!
reconfigurePins
	digitalPins values , analogPins values
		do: [:each | each refresh]! !

!Arduino methodsFor: 'keeping alive' stamp: 'Richo 12/11/2012 13:28'!
sendKeepAlive
	self isConnected ifFalse: [self stopKeepAliveProcess].
	protocol sendKeepAlive! !

!Arduino methodsFor: 'keeping alive' stamp: 'Richo 12/11/2012 13:26'!
startKeepAliveProcess
	keepAliveProcess notNil
		ifTrue: [^ self].
	keepAliveProcess := [[self sendKeepAlive.
			(Delay forMilliseconds: self keepAliveTime) wait] repeat]
				forkNamed: 'KeepAlive: ', self asString! !

!Arduino methodsFor: 'keeping alive' stamp: 'Richo 12/11/2012 13:43'!
stopKeepAliveProcess
	"In order to correctly terminate the process and set its variable
	to nil, we need to store it in a temp variable, so that we set the 
	variable to nil before terminating the process. Since we're in the
	same process if we terminate it before this won't get executed"
	| p |
	keepAliveProcess isNil
		ifTrue: [^ self].
	p := keepAliveProcess.
	keepAliveProcess := nil.
	p terminate! !

!Arduino methodsFor: 'accessing' stamp: 'Richo 11/23/2012 12:54'!
typeClass
	^ ArduinoType! !

Object subclass: #ArduinoPin
	instanceVariableNames: 'number mode value arduino'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core'!

!ArduinoPin commentStamp: 'Richo 9/15/2009 11:49' prior: 0!
Abstract class. I represent an ArduinoPin!

ArduinoPin subclass: #AFMotorPin
	instanceVariableNames: 'state direction speed previousState'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-AFMotorShield'!

!AFMotorPin methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:16'!
direction
	^ direction
! !

!AFMotorPin methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:16'!
direction: s 
	direction := s.
	self refresh
! !

!AFMotorPin methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:16'!
initialize
	state := #Stopped.
	previousState := #Stopped.
	direction := #Forward.
	speed := 100
! !

!AFMotorPin methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:16'!
refresh
	state = #Running
		ifTrue: [previousState := #Running.
			self protocol
				runMotor: number
				command: direction
				speed: speed]
		ifFalse: [previousState = #Running
				ifTrue: [previousState = #Stopped.
					self protocol runMotor: number command: #Stop]]
! !

!AFMotorPin methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:16'!
speed
	^ speed
! !

!AFMotorPin methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:16'!
speed: s 
	speed := s.
	self refresh
! !

!AFMotorPin methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:16'!
state
	^ state
! !

!AFMotorPin methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:17'!
state: s 
	state := s.
	self refresh
! !

ArduinoPin subclass: #ArduinoAnalogPin
	instanceVariableNames: 'isActive'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core'!

!ArduinoAnalogPin commentStamp: 'Richo 9/15/2009 11:52' prior: 0!
I represent an Arduino analog pin. Analog pins are always input. Before I can access my value, I must be activated by sending #activate.!

!ArduinoAnalogPin methodsFor: 'activation' stamp: 'Richo 8/31/2011 15:29'!
activate
	isActive := true.
	self refresh! !

!ArduinoAnalogPin methodsFor: 'configuring devices' stamp: 'Richo 9/6/2011 11:50'!
configure: aDevice
	aDevice configureAnalog: self! !

!ArduinoAnalogPin methodsFor: 'activation' stamp: 'Richo 8/31/2011 15:29'!
deactivate
	isActive := false.
	self refresh! !

!ArduinoAnalogPin methodsFor: 'configuring devices' stamp: 'Richo 9/6/2011 11:56'!
deleteConfiguration: aDevice
	aDevice deleteConfigurationAnalog: self! !

!ArduinoAnalogPin methodsFor: 'initializing' stamp: 'Richo 9/7/2011 10:54'!
initialize
	super initialize.
	mode := InputMode new.
	isActive := false.! !

!ArduinoAnalogPin methodsFor: 'activation' stamp: 'Richo 8/31/2009 15:24'!
isActive
^isActive! !

!ArduinoAnalogPin methodsFor: 'testing' stamp: 'Richo 9/6/2011 12:14'!
isAnalog
	^ true! !

!ArduinoAnalogPin methodsFor: 'initializing' stamp: 'Richo 10/24/2011 14:01'!
refresh
	self refreshMode; refreshValue.
	isActive
		ifTrue: [self protocol activateAnalogPin: number]
		ifFalse: [self protocol deactivateAnalogPin: number] ! !

!ArduinoAnalogPin methodsFor: 'mode' stamp: 'Richo 10/25/2011 16:10'!
refreshMode
	mode refreshModeOn: 14 + number using: self protocol.! !

!ArduinoAnalogPin methodsFor: 'value' stamp: 'Richo 10/25/2011 16:10'!
refreshValue
	mode isOutput
		ifTrue: [mode
				refreshValue: value
				on: 14 + number
				using: self protocol]! !

!ArduinoAnalogPin methodsFor: 'value' stamp: 'Richo 10/24/2011 14:01'!
value
	^ mode isOutput
		ifTrue: [value]
		ifFalse: [value := self protocol analogRead: number]
! !

ArduinoPin subclass: #ArduinoDigitalPin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core'!

!ArduinoDigitalPin commentStamp: 'Richo 9/15/2009 11:55' prior: 0!
I represent an Arduino digital pin. Digital pins can be input or output. To change my mode you must send #mode: with a PinMode as a parameter or you can use #setInput/#setOutput.!

!ArduinoDigitalPin methodsFor: 'configuring devices' stamp: 'Richo 9/6/2011 11:50'!
configure: aDevice
	aDevice configureDigital: self! !

!ArduinoDigitalPin methodsFor: 'configuring devices' stamp: 'Richo 9/6/2011 11:55'!
deleteConfiguration: aDevice
	aDevice deleteConfigurationDigital: self! !

!ArduinoDigitalPin methodsFor: 'initialize-release' stamp: 'Richo 9/7/2011 10:54'!
initialize
	super initialize.
	mode := OutputMode new.
! !

!ArduinoDigitalPin methodsFor: 'testing' stamp: 'Richo 9/6/2011 12:13'!
isDigital
	^ true! !

!ArduinoDigitalPin methodsFor: 'initialize-release' stamp: 'Richo 9/7/2011 10:54'!
refresh
	self refreshMode; refreshValue! !

!ArduinoDigitalPin methodsFor: 'mode' stamp: 'Richo 10/24/2011 14:01'!
refreshMode
	mode refreshModeOn: number using: self protocol.! !

!ArduinoDigitalPin methodsFor: 'value' stamp: 'Richo 10/24/2011 14:01'!
refreshValue
	mode isOutput
		ifTrue: [mode
				refreshValue: value
				on: number
				using: self protocol]! !

!ArduinoDigitalPin methodsFor: 'value' stamp: 'Richo 10/24/2011 14:01'!
value 
	^ mode isOutput
		ifTrue: [value]
		ifFalse: [value := self protocol digitalRead: number]! !

!ArduinoPin class methodsFor: 'instance creation' stamp: 'Richo 9/7/2011 11:02'!
new
	self shouldNotImplement! !

!ArduinoPin class methodsFor: 'instance creation' stamp: 'Richo 10/24/2011 14:00'!
on: anArduino number: pinNumber 
	^ self basicNew initializeWith: anArduino number: pinNumber! !

!ArduinoPin methodsFor: 'configuring devices' stamp: 'Richo 9/6/2011 11:49'!
configure: aDevice
	self subclassResponsibility ! !

!ArduinoPin methodsFor: 'configuring devices' stamp: 'Richo 9/6/2011 11:55'!
deleteConfiguration: aDevice
	self subclassResponsibility ! !

!ArduinoPin methodsFor: 'initializing' stamp: 'Richo 10/24/2011 14:01'!
initializeWith: anArduino number: aSmallInteger
	self initialize.
	arduino := anArduino.
	number := aSmallInteger.
	value := 0! !

!ArduinoPin methodsFor: 'testing' stamp: 'Richo 9/6/2011 12:13'!
isAnalog
	^ false! !

!ArduinoPin methodsFor: 'testing' stamp: 'Richo 9/6/2011 12:13'!
isDigital
	^ false! !

!ArduinoPin methodsFor: 'mode' stamp: 'Richo 9/7/2011 10:52'!
mode
	^ mode! !

!ArduinoPin methodsFor: 'mode' stamp: 'Richo 9/7/2011 10:52'!
mode: aPinMode
	mode := aPinMode.
	self refreshMode! !

!ArduinoPin methodsFor: 'accessing' stamp: 'Richo 8/24/2009 16:51'!
number
^number! !

!ArduinoPin methodsFor: 'printing' stamp: 'Richo 8/19/2009 14:18'!
printOn: aStream
super printOn: aStream.
aStream nextPutAll: '(', number asString, ')'! !

!ArduinoPin methodsFor: 'accessing' stamp: 'Richo 10/24/2011 14:02'!
protocol
	^ arduino protocol! !

!ArduinoPin methodsFor: 'initializing' stamp: 'Richo 9/7/2011 10:59'!
refresh
	self subclassResponsibility! !

!ArduinoPin methodsFor: 'mode' stamp: 'Richo 9/7/2011 10:53'!
refreshMode
	self subclassResponsibility ! !

!ArduinoPin methodsFor: 'value' stamp: 'Richo 9/7/2011 10:55'!
refreshValue
	self subclassResponsibility ! !

!ArduinoPin methodsFor: 'mode' stamp: 'Richo 9/7/2011 10:53'!
setInput
	self mode: InputMode new! !

!ArduinoPin methodsFor: 'mode' stamp: 'Richo 9/7/2011 10:53'!
setOutput
	self mode: OutputMode new! !

!ArduinoPin methodsFor: 'value' stamp: 'Richo 12/29/2011 17:35'!
value: aNumber 
	value := aNumber rounded.
	self refreshValue! !

Object subclass: #ArduinoProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core'!

!ArduinoProtocol commentStamp: 'Richo 3/9/2010 10:34' prior: 0!
Abstract class. I define the protocol to communicate with an Arduino board.!

ArduinoProtocol subclass: #ArduinoDirectMode
	instanceVariableNames: 'port steppingProcess digitalPins analogPins argsToRead queue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Firmware'!

!ArduinoDirectMode class methodsFor: 'constants-request commands' stamp: 'Richo 8/24/2011 09:38'!
rqActivateAnalogPin
	^ 0! !

!ArduinoDirectMode class methodsFor: 'constants-request commands' stamp: 'Richo 8/24/2011 09:38'!
rqActivateDigitalPort
	^ 2! !

!ArduinoDirectMode class methodsFor: 'constants-request commands' stamp: 'Richo 8/24/2011 09:38'!
rqAnalogWrite
	^ 1! !

!ArduinoDirectMode class methodsFor: 'constants-request commands' stamp: 'Richo 8/24/2011 09:38'!
rqAttachServo
	^ 5! !

!ArduinoDirectMode class methodsFor: 'constants-request commands' stamp: 'Richo 12/29/2011 17:27'!
rqDetachServo
	^ 7! !

!ArduinoDirectMode class methodsFor: 'constants-request commands' stamp: 'Richo 8/24/2011 09:38'!
rqDigitalPinMode
	^ 3! !

!ArduinoDirectMode class methodsFor: 'constants-request commands' stamp: 'Richo 8/24/2011 09:38'!
rqDigitalWrite
	^ 4! !

!ArduinoDirectMode class methodsFor: 'constants-request commands' stamp: 'Richo 1/7/2013 15:47'!
rqDisconnect
	^ 8! !

!ArduinoDirectMode class methodsFor: 'constants-request commands' stamp: 'Richo 1/7/2013 16:19'!
rqKeepAlive
	^ 64! !

!ArduinoDirectMode class methodsFor: 'constants-request commands' stamp: 'Richo 1/18/2013 18:18'!
rqPlayTone
	^ 9! !

!ArduinoDirectMode class methodsFor: 'constants-request commands' stamp: 'Richo 8/24/2011 09:39'!
rqServoAngle
	^ 6! !

!ArduinoDirectMode class methodsFor: 'constants-response commands' stamp: 'Richo 8/30/2011 16:43'!
rsAnalogPin
	^ 2! !

!ArduinoDirectMode class methodsFor: 'constants-response commands' stamp: 'Richo 8/30/2011 16:43'!
rsDigitalPort
	^ 1! !

!ArduinoDirectMode class methodsFor: 'constants-response commands' stamp: 'Richo 8/30/2011 16:43'!
rsVersion
	^ 3! !

!ArduinoDirectMode methodsFor: 'analog pins' stamp: 'Richo 8/30/2011 16:55'!
activateAnalogPin: pinNumber
	self send: {self asCommand: self class rqActivateAnalogPin.
				self asArgument: pinNumber.
				self asArgument: 1} asByteArray.
! !

!ArduinoDirectMode methodsFor: 'digital pins' stamp: 'Richo 8/30/2011 12:03'!
activateDigitalPort: portIndex
	self send: {self asCommand: self class rqActivateDigitalPort.
				self asArgument: portIndex.
				self asArgument: 1} asByteArray.
! !

!ArduinoDirectMode methodsFor: 'analog pins' stamp: 'Richo 8/29/2011 16:45'!
analogRead: pinNumber
	^ analogPins at: pinNumber! !

!ArduinoDirectMode methodsFor: 'digital pins' stamp: 'Richo 12/29/2011 17:41'!
analogWrite: pinIndex value: pinValue
	self send: {self asCommand: self class rqAnalogWrite.
				self asArgument: pinIndex.
				self asArgument: (pinValue bitAnd: 2r01111111).
				self asArgument: (pinValue bitAnd: 2r10000000) >> 7} asByteArray! !

!ArduinoDirectMode methodsFor: 'converting' stamp: 'Richo 8/29/2011 12:47'!
asArgument: byte
	^ byte bitOr: 2r10000000! !

!ArduinoDirectMode methodsFor: 'converting' stamp: 'Richo 8/29/2011 11:17'!
asCommand: byte
	^ byte! !

!ArduinoDirectMode methodsFor: 'servos' stamp: 'Richo 12/29/2011 17:28'!
attachServoToPin: pinNumber
	self send: {self asCommand: self class rqAttachServo.
				self asArgument: pinNumber} asByteArray.
! !

!ArduinoDirectMode methodsFor: 'connection' stamp: 'Richo 12/11/2012 13:26'!
connectOnPort: aString
	"If the port cant't be open it stores nil, this way we can know if it is
	connected by asking if port is nil.
	In windows we can't open a port using the port name, so I convert the
	string to an integer"
	self isConnected ifTrue: [^ self].
	port := SerialPort new baudRate: 57600.
	port := port
				openPort: (('COM*' match: aString)
						ifTrue: [aString asInteger]
						ifFalse: [aString]).
	self isConnected
		ifTrue: [self controlFirmwareInstalled;
					startSteppingProcess]! !

!ArduinoDirectMode methodsFor: 'connection' stamp: 'Richo 8/24/2011 11:12'!
connectOnPort: aString boardType: anArduinoType
	"Ignore board type"
	self connectOnPort: aString! !

!ArduinoDirectMode methodsFor: 'connection' stamp: 'Richo 12/29/2011 13:48'!
controlFirmwareInstalled
	| time timeout |
	time := Time millisecondClockValue.
	timeout := 5000.
	[100 milliSeconds asDelay wait.
	('*HOLA, SOY ARDUINO*' match: port readString)
		ifTrue: [
			port nextPutAll: (ByteArray with: 255).
			[port readString = ''] whileFalse.
			^ self]]
		doWhileTrue: [Time millisecondClockValue - time < timeout].
	^ FirmwareNotInstalled signal! !

!ArduinoDirectMode methodsFor: 'analog pins' stamp: 'Richo 8/30/2011 16:55'!
deactivateAnalogPin: pinNumber
	self send: {self asCommand: self class rqActivateAnalogPin.
				self asArgument: pinNumber.
				self asArgument: 0} asByteArray.
! !

!ArduinoDirectMode methodsFor: 'digital pins' stamp: 'Richo 8/30/2011 12:02'!
deactivateDigitalPort: portIndex
	self send: {self asCommand: self class rqActivateDigitalPort.
				self asArgument: portIndex.
				self asArgument: 0} asByteArray.
! !

!ArduinoDirectMode methodsFor: 'servos' stamp: 'Richo 12/29/2011 17:28'!
detachServoFromPin: pinNumber
	self send: {self asCommand: self class rqDetachServo.
				self asArgument: pinNumber} asByteArray.
! !

!ArduinoDirectMode methodsFor: 'digital pins' stamp: 'Richo 8/30/2011 17:23'!
digitalPin: pinIndex mode: pinMode
	"Digital ports should always be active"
	pinIndex <= 7
		ifTrue: [self activateDigitalPort: 0]
		ifFalse: [self activateDigitalPort: 1].
	self send: {self asCommand: self class rqDigitalPinMode.
				self asArgument: pinIndex.
				self asArgument: pinMode} asByteArray.
! !

!ArduinoDirectMode methodsFor: 'digital pins' stamp: 'Richo 8/30/2011 17:24'!
digitalRead: pinNumber
	^ digitalPins at: pinNumber! !

!ArduinoDirectMode methodsFor: 'digital pins' stamp: 'Richo 8/30/2011 12:03'!
digitalWrite: pinIndex value: pinValue
	self send: {self asCommand: self class rqDigitalWrite.
				self asArgument: pinIndex.
				self asArgument: pinValue} asByteArray! !

!ArduinoDirectMode methodsFor: 'connection' stamp: 'Richo 1/16/2013 17:58'!
disconnect
	self isConnected ifFalse: [^ self].
	[self sendDisconnect] on: Error do: ["Nothing"].
	self privateDisconnect! !

!ArduinoDirectMode methodsFor: 'receiving' stamp: 'Richo 8/30/2011 12:42'!
executeAnalogPin
	| pin value1 value2 |
	pin := queue pop.
	value1 := queue pop.
	value2 := queue pop.
	analogPins at: pin put: (value1 bitOr: (value2 << 7))! !

!ArduinoDirectMode methodsFor: 'receiving' stamp: 'Richo 8/30/2011 17:19'!
executeCommand
	| command |
	command := queue pop.
	command caseOf: {
		[self class rsDigitalPort] -> [self executeDigitalPort].
		[self class rsAnalogPin] -> [self executeAnalogPin].
		[self class rsVersion] -> [self executeVersion]
	} otherwise: [Transcript show: ('Invalid command: {1}' format:  {command}); cr].
	argsToRead := -1.! !

!ArduinoDirectMode methodsFor: 'receiving' stamp: 'Richo 8/30/2011 17:24'!
executeDigitalPort
	| digitalPort value |
	digitalPort := queue pop.
	value := queue pop.
	digitalPins at: 2 + (6 * digitalPort) put: (value bitAnd: 2r1).
	digitalPins at: 3 + (6 * digitalPort) put: ((value >> 1) bitAnd: 2r1).
	digitalPins at: 4 + (6 * digitalPort) put: ((value >> 2) bitAnd: 2r1).
	digitalPins at: 5 + (6 * digitalPort) put: ((value >> 3) bitAnd: 2r1).
	digitalPins at: 6 + (6 * digitalPort) put: ((value >> 4) bitAnd: 2r1).
	digitalPins at: 7 + (6 * digitalPort) put: ((value >> 5) bitAnd: 2r1)! !

!ArduinoDirectMode methodsFor: 'converting' stamp: 'Richo 8/29/2011 12:48'!
getArgument: byte
	^ byte bitAnd: 2r01111111! !

!ArduinoDirectMode methodsFor: 'converting' stamp: 'Richo 8/29/2011 12:49'!
getCommand: byte
	^ byte! !

!ArduinoDirectMode methodsFor: 'initialize-release' stamp: 'Richo 12/11/2012 13:16'!
initialize
	super initialize.
	digitalPins := self newDictionaryFrom: 2 to: 13 withAll: 0.
	analogPins := self newDictionaryFrom: 0 to: 5 withAll: 0.
	queue := Queue new.
	argsToRead := -1! !

!ArduinoDirectMode methodsFor: 'converting' stamp: 'Richo 8/29/2011 12:48'!
isArgument: byte
	^ byte >> 7 = 1! !

!ArduinoDirectMode methodsFor: 'converting' stamp: 'Richo 8/29/2011 12:48'!
isCommand: byte
	^ byte >> 7 = 0! !

!ArduinoDirectMode methodsFor: 'connection' stamp: 'Richo 8/23/2011 12:56'!
isConnected
	^ port notNil! !

!ArduinoDirectMode methodsFor: 'keeping alive' stamp: 'Richo 12/11/2012 13:16'!
keepAliveTime
	"Every second"
	^ 1000! !

!ArduinoDirectMode methodsFor: 'accessing' stamp: 'Richo 12/23/2011 16:16'!
nameSymbol
	^ #direct! !

!ArduinoDirectMode methodsFor: 'initialize-release' stamp: 'Richo 8/23/2011 14:53'!
newDictionaryFrom: start to: stop withAll: value
	| dict |
	dict := Dictionary new.
	start to: stop do: [:each |
		dict at: each put: value].
	^ dict
! !

!ArduinoDirectMode methodsFor: 'sound' stamp: 'Richo 1/18/2013 18:19'!
playTone: frequency duration: duration pin: pin
	| freq dur |
	freq := frequency rounded.
	dur := duration rounded.
	self send: {self asCommand: self class rqPlayTone.
				"pin"
				self asArgument: pin.
				"frequency"
				self asArgument: (freq bitAnd: 2r1111111).
				self asArgument: ((freq bitAnd: 2r11111110000000) >> 7).
				self asArgument: ((freq bitAnd: 2r111111100000000000000) >> 14) .
				"duration"
				self asArgument: (dur bitAnd: 2r1111111).
				self asArgument: ((dur bitAnd: 2r11111110000000) >> 7)} asByteArray! !

!ArduinoDirectMode methodsFor: 'connection' stamp: 'Richo 1/7/2013 15:58'!
privateDisconnect
	"Use #disconnect"
	self stopSteppingProcess.
	port close.
	port := nil.
	self initialize! !

!ArduinoDirectMode methodsFor: 'receiving' stamp: 'Richo 12/11/2012 13:13'!
processInput
	| data |
	data := port readByteArray.
	data 
		do: [:inByte |
			(self isCommand: inByte)
				ifTrue: [self readCommand: inByte].
			(self isArgument: inByte)
				ifTrue: [self readArgument: inByte].

			argsToRead = 0
				ifTrue: [self executeCommand]]! !

!ArduinoDirectMode methodsFor: 'receiving' stamp: 'Richo 8/29/2011 12:45'!
readArgument: inByte
	| argument |
	argument := self getArgument: inByte.
	queue push: argument.
	argsToRead := argsToRead - 1! !

!ArduinoDirectMode methodsFor: 'receiving' stamp: 'Richo 8/30/2011 16:57'!
readCommand: inByte
	| command |
	queue flush.
	command := self getCommand: inByte.
	queue push: command.
	self setArgsToReadFor: command.! !

!ArduinoDirectMode methodsFor: 'sending' stamp: 'Richo 1/7/2013 15:59'!
send: aByteArray
	self isConnected ifFalse: [^ self].
	"Added a small delay because /dev/ttyACM* devices seem
	to fail when sending a lot of small messages"
	1 milliSecond asDelay wait.
	(port nextPutAll: aByteArray) < 0
		ifTrue: [
			"An error occurred, maybe the board was unplugged.
			Just in case, disconnect"
			self privateDisconnect]! !

!ArduinoDirectMode methodsFor: 'connection' stamp: 'Richo 1/7/2013 15:55'!
sendDisconnect
	self send: {self asCommand: self class rqDisconnect} asByteArray.! !

!ArduinoDirectMode methodsFor: 'keeping alive' stamp: 'Richo 12/11/2012 13:32'!
sendKeepAlive
	self send: {self asCommand: self class rqKeepAlive} asByteArray! !

!ArduinoDirectMode methodsFor: 'servos' stamp: 'Richo 12/29/2011 17:39'!
servoOnPin: pinNumber angle: degrees
	self send: {self asCommand: self class rqServoAngle.
				self asArgument: pinNumber.
				self asArgument: (degrees bitAnd: 2r01111111).
				self asArgument: (degrees bitAnd: 2r10000000) >> 7} asByteArray.
! !

!ArduinoDirectMode methodsFor: 'receiving' stamp: 'Richo 8/29/2011 17:58'!
setArgsToReadFor: command
	command caseOf: {
		[self class rsDigitalPort] -> [argsToRead := 2].
		[self class rsAnalogPin] -> [argsToRead := 3]
	} otherwise: [Transcript show: 'Error - command: ', command asString; cr]
! !

!ArduinoDirectMode methodsFor: 'stepping' stamp: 'Richo 8/23/2011 12:56'!
startSteppingProcess
	steppingProcess notNil
		ifTrue: [^ self].
	steppingProcess := [[self step.
			(Delay forMilliseconds: self stepTime) wait] repeat]
				forkNamed: self asString ! !

!ArduinoDirectMode methodsFor: 'stepping' stamp: 'Richo 12/11/2012 13:14'!
step
	[self processInput]
		on: Error
		do: [:err | 
			Transcript show: self asString , ': ' , err description; cr. 
			self initialize]! !

!ArduinoDirectMode methodsFor: 'stepping' stamp: 'Richo 8/29/2011 16:04'!
stepTime
	^ 1! !

!ArduinoDirectMode methodsFor: 'stepping' stamp: 'Richo 8/23/2011 12:57'!
stopSteppingProcess
	steppingProcess isNil
		ifTrue: [^ self].
	steppingProcess terminate. 
	steppingProcess := nil! !

!ArduinoDirectMode methodsFor: 'pin modes' stamp: 'Richo 8/24/2011 11:14'!
valueForInputMode
	"Return the internal (usually numeric) value used to represent the input mode"
	^ 0! !

!ArduinoDirectMode methodsFor: 'pin modes' stamp: 'Richo 8/24/2011 11:14'!
valueForOutputMode
	"Return the internal (usually numeric) value used to represent the output mode"
	^ 1! !

!ArduinoDirectMode methodsFor: 'pin modes' stamp: 'Richo 8/24/2011 11:14'!
valueForPwmMode
"Return the internal (usually numeric) value used to represent the pwm mode"
	^ 2! !

!ArduinoDirectMode methodsFor: 'pin modes' stamp: 'Richo 8/24/2011 11:14'!
valueForServoMode
"Return the internal (usually numeric) value used to represent the servo mode"
	^ 3! !

!ArduinoProtocol methodsFor: 'analog pins' stamp: 'Richo 8/26/2009 16:51'!
activateAnalogPin: pinNumber
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'digital pins' stamp: 'Richo 9/11/2009 13:46'!
activateDigitalPort: portNumber
self subclassResponsibility
! !

!ArduinoProtocol methodsFor: 'analog pins' stamp: 'Richo 8/26/2009 16:09'!
analogRead: pinNumber
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'servos' stamp: 'Richo 8/26/2009 16:54'!
attachServoToPin: pinNumber
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'connection' stamp: 'Richo 1/31/2011 10:00'!
connectOnPort: portName boardType: anArduinoType
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'analog pins' stamp: 'Richo 8/26/2009 16:51'!
deactivateAnalogPin: pinNumber
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'digital pins' stamp: 'Richo 9/11/2009 13:47'!
deactivateDigitalPort: portNumber
self subclassResponsibility
! !

!ArduinoProtocol methodsFor: 'servos' stamp: 'Richo 9/7/2009 16:25'!
detachServoFromPin: pinNumber
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'digital pins' stamp: 'Richo 8/26/2009 16:53'!
digitalPin: pinNumber mode: mode
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'digital pins' stamp: 'Richo 8/26/2009 16:51'!
digitalRead: pinNumber
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'digital pins' stamp: 'Richo 8/26/2009 17:06'!
digitalWrite: pinNumber value: value
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'connection' stamp: 'Richo 8/26/2009 16:08'!
disconnect
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'testing' stamp: 'Richo 12/23/2011 15:10'!
isCompiled
	^ false! !

!ArduinoProtocol methodsFor: 'connection' stamp: 'Richo 8/26/2009 16:09'!
isConnected
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'keep alive' stamp: 'Richo 12/11/2012 13:28'!
keepAliveTime
	^ 100000! !

!ArduinoProtocol methodsFor: 'sound' stamp: 'Richo 1/18/2013 18:20'!
playTone: frequency duration: duration pin: pin
	self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'keep alive' stamp: 'Richo 12/11/2012 13:27'!
sendKeepAlive
	"Do nothing"! !

!ArduinoProtocol methodsFor: 'servos' stamp: 'Richo 8/26/2009 16:54'!
servoOnPin: pinNumber angle: degrees
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'pin modes' stamp: 'Richo 3/9/2010 10:52'!
valueForInputMode
"Return the internal (usually numeric) value used to represent the input mode"
^self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'pin modes' stamp: 'Richo 3/9/2010 10:53'!
valueForOutputMode
"Return the internal (usually numeric) value used to represent the output mode"
^self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'pin modes' stamp: 'Richo 4/30/2010 15:44'!
valueForPwmMode
"Return the internal (usually numeric) value used to represent the pwm mode"
^self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'pin modes' stamp: 'Richo 3/9/2010 10:53'!
valueForServoMode
"Return the internal (usually numeric) value used to represent the servo mode"
^self subclassResponsibility! !

ArduinoProtocol subclass: #Firmata
	instanceVariableNames: 'port analogInputData digitalInputData digitalOutputData parsingSysex storedInputData sysexBytesRead waitForData executeMultiByteCommand multiByteChannel majorVersion minorVersion steppingProcess'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Firmata'!

!Firmata commentStamp: 'Richo 3/9/2010 11:03' prior: 0!
I am an implementation of the Firmata protocol for talking to an Arduino board. 
For more information check: http://www.firmata.org/

This implementation is mostly based on FirmataVB by Andrew Craigie.
http://www.acraigie.com/programming/firmatavb/default.html!

!Firmata class methodsFor: 'instance creation' stamp: 'Richo 1/31/2011 09:09'!
onPort: portNumber baudRate: aNumber
^self new connectOnPort: portNumber baudRate: aNumber! !

!Firmata methodsFor: 'analog pins' stamp: 'Richo 1/11/2010 18:36'!
activateAnalogPin: pinNumber 
	self analogPinReport: pinNumber onOff: 1.
! !

!Firmata methodsFor: 'digital pins' stamp: 'Richo 12/11/2012 14:02'!
activateDigitalPort: n
	self digitalPortReport: n onOff: 1
! !

!Firmata methodsFor: 'analog pins' stamp: 'Richo 1/14/2013 19:28'!
analogPinReport: pin onOff: onOff 
	"Sends a message to turn Analog Pin reportng on or off for a pin"
	self port 
		nextPutAll: (ByteArray
				with: (FirmataConstants reportAnalog  bitOr: pin)
				with: onOff)! !

!Firmata methodsFor: 'analog pins' stamp: 'Richo 8/18/2009 12:35'!
analogRead: pinNumber
"Returns an analog value for an individual Analog pin"
^analogInputData at: pinNumber + 1! !

!Firmata methodsFor: 'digital pins' stamp: 'Richo 8/25/2009 14:47'!
analogWrite: pin value: value 
	"Sends a PWM value to a digital pin. 0 - 255"
	| analogWriteBuffer |
	analogWriteBuffer := ByteArray new: 3 withAll: 0.
	analogWriteBuffer
		at: 1
		put: (FirmataConstants analogMessage
				bitOr: (pin bitAnd: 15));
		
		at: 2
		put: (value rounded bitAnd: 127);
		
		at: 3
		put: ((value rounded bitShift: -7)
				bitAnd: 127).
self port  nextPutAll: analogWriteBuffer! !

!Firmata methodsFor: 'servos' stamp: 'Richo 9/1/2009 11:35'!
attachServoToPin: pinNumber 
	self
		servoConfig: pinNumber
		minPulse: 544
		maxPulse: 2400
		angle: 0! !

!Firmata methodsFor: 'connection' stamp: 'Richo 12/23/2011 17:23'!
connectOnPort: aString boardType: anArduinoType
	"If the port cant't be open it stores nil, this way we can know if it is
	connected by asking if port is nil.
	In windows we can't open a port using the port name, so I convert the
	string to an integer"
	self isConnected ifTrue: [^ self].
	port := SerialPort new baudRate: anArduinoType firmataRate.
	port := port
				openPort: (('COM*' match: aString)
						ifTrue: [aString asInteger]
						ifFalse: [aString]).
	self isConnected
		ifTrue: [self startSteppingProcess; controlFirmataInstallation]! !

!Firmata methodsFor: 'error control' stamp: 'Richo 4/6/2011 10:41'!
controlConnection
self isConnected ifFalse: [^self error: 'Serial port is not connected' translated].! !

!Firmata methodsFor: 'error control' stamp: 'Richo 12/23/2011 17:13'!
controlFirmataInstallation
	"If Firmata is not installed I throw an error"
	self isFirmataInstalled
		ifFalse: [^ self error: 'Firmata doesn''t seem to be installed.
Please use the Arduino IDE to upload the StandardFirmata sketch' translated]! !

!Firmata methodsFor: 'analog pins' stamp: 'Richo 8/26/2009 17:03'!
deactivateAnalogPin: pinNumber 
	self analogPinReport: pinNumber onOff: 0! !

!Firmata methodsFor: 'digital pins' stamp: 'Richo 9/11/2009 13:46'!
deactivateDigitalPort: n
			self digitalPortReport: n onOff: 0
! !

!Firmata methodsFor: 'servos' stamp: 'Richo 9/7/2009 16:45'!
detachServoFromPin: pinNumber 
	self pin: pinNumber mode: FirmataConstants pinModeOutput ! !

!Firmata methodsFor: 'digital pins' stamp: 'Richo 1/14/2013 19:27'!
digitalPin: pinNumber mode: mode
	"Digital ports should always be active"
	pinNumber <= 7
		ifTrue: [self activateDigitalPort: 0]
		ifFalse: [self activateDigitalPort: 1].
	(pinNumber > 13 and: [mode = 0])
		ifTrue: ["Analog pins should not be set as INPUT but ANALOG"
			self pin: pinNumber mode: FirmataConstants pinModeAnalog]
		ifFalse: [self pin: pinNumber mode: mode]! !

!Firmata methodsFor: 'digital pins' stamp: 'Richo 8/31/2009 16:22'!
digitalPortReport: port1 onOff: onOff
"Sends a message to toggle reporting for a whole digital port"
	self port 
		nextPutAll: (ByteArray
				with: (FirmataConstants reportDigital bitOr: port1 )
				with: onOff)! !

!Firmata methodsFor: 'digital pins' stamp: 'Richo 9/14/2009 15:32'!
digitalRead: pinNumber 
	"Retuns On/Off data for an individual Digital Pin"
	^ ((digitalInputData at: (pinNumber bitShift: -3)
			+ 1)
		bitShift: (pinNumber bitAnd: 7) * -1)
		bitAnd: 1! !

!Firmata methodsFor: 'digital pins' stamp: 'Richo 1/6/2012 13:29'!
digitalWrite: pin value: value 
	"Sends an On or Off message to an individual Digital Pin"
	| portNumber adjustment digitalWriteBuffer |
	portNumber := (pin bitShift: -3)
				bitAnd: 15.
	adjustment := 1
				bitShift: (pin bitAnd: 7).
	digitalWriteBuffer := ByteArray new: 3 withAll: 0.
	value = 0
		ifTrue: [digitalOutputData
				at: portNumber + 1
				put: ((digitalOutputData at: portNumber + 1)
						bitAnd: adjustment bitInvert)]
		ifFalse: [digitalOutputData
				at: portNumber + 1
				put: ((digitalOutputData at: portNumber + 1)
						bitOr: adjustment)].
	digitalWriteBuffer
		at: 1
		put: (FirmataConstants digitalMessage bitOr: portNumber);
		at: 2
		put: ((digitalOutputData at: portNumber + 1)
				bitAnd: 127);
		at: 3
		put: ((digitalOutputData at: portNumber + 1)
				bitShift: -7).
	self port nextPutAll: digitalWriteBuffer! !

!Firmata methodsFor: 'connection' stamp: 'Richo 3/9/2010 11:12'!
disconnect
	self isConnected
		ifTrue: [self stopSteppingProcess.
			port close.
			port := nil.
			self initializeVariables]! !

!Firmata methodsFor: 'other commands' stamp: 'Richo 8/18/2009 12:41'!
endSysex
"Sends an End Sysex message"
self port  nextPutAll: (ByteArray with: FirmataConstants endSysex  ).! !

!Firmata methodsFor: 'initialize-release' stamp: 'Richo 9/8/2009 12:40'!
initialize
self initializeVariables! !

!Firmata methodsFor: 'initialize-release' stamp: 'Richo 9/8/2009 12:40'!
initializeVariables
	digitalOutputData := Array new: 16 withAll: 0.
	digitalInputData := Array new: 16 withAll: 0.
	analogInputData := Array new: 16 withAll: 0.
	storedInputData := Array new: FirmataConstants maxDataBytes + 1 withAll: 0.
	parsingSysex := false.
	sysexBytesRead := waitForData := executeMultiByteCommand := multiByteChannel := majorVersion := minorVersion := 0! !

!Firmata methodsFor: 'connection' stamp: 'Richo 8/18/2009 12:40'!
isConnected
^port notNil! !

!Firmata methodsFor: 'testing' stamp: 'Richo 1/31/2011 11:53'!
isFirmataInstalled
"I check if Firmata responds well to queryVersion. I try for 5 seconds"
| t timeout |
t := Time millisecondClockValue.
timeout := 5000.
[self queryVersion. 500 milliSeconds asDelay wait.
self version > 0 or: [(Time millisecondsSince: t) > timeout]] whileFalse.
^self version > 0! !

!Firmata methodsFor: 'keep alive' stamp: 'Richo 12/11/2012 14:05'!
keepAliveTime
	^ 1000! !

!Firmata methodsFor: 'accessing' stamp: 'Richo 1/5/2010 13:02'!
majorVersion
^majorVersion! !

!Firmata methodsFor: 'accessing' stamp: 'Richo 8/18/2009 13:22'!
majorVersion: version1 minorVersion: version2
majorVersion := version1.
minorVersion := version2! !

!Firmata methodsFor: 'accessing' stamp: 'Richo 1/5/2010 13:02'!
minorVersion
^minorVersion! !

!Firmata methodsFor: 'accessing' stamp: 'Richo 12/23/2011 16:42'!
nameSymbol
	^ #firmata! !

!Firmata methodsFor: 'receiving' stamp: 'Richo 3/9/2010 11:34'!
parseCommandHeader: inputByte 
"I check the inputByte for a command header and then I set the variables accordingly to process the data in the following step"
	| command |
	inputByte < 240
		ifTrue: [command := inputByte bitAnd: 240.
			multiByteChannel := inputByte bitAnd: 15.]
		ifFalse: [command := inputByte].
	command caseOf: {
		[FirmataConstants analogMessage] -> 
			[waitForData := 2.
			executeMultiByteCommand := command].
		[FirmataConstants digitalMessage] -> 
			[waitForData := 2.
			executeMultiByteCommand := command].
		[FirmataConstants setPinMode] -> 
			[waitForData := 2.
			executeMultiByteCommand := command].
		[FirmataConstants reportDigital] -> 
			[waitForData := 1.
			executeMultiByteCommand := command].
		[FirmataConstants startSysex] -> 
			[parsingSysex := true.
			sysexBytesRead := 0].
		[FirmataConstants reportVersion] -> 
			[waitForData := 2.
			executeMultiByteCommand := command]}
		 otherwise: []! !

!Firmata methodsFor: 'receiving' stamp: 'Richo 10/25/2011 15:59'!
parseData: inputByte 
"I check inputByte for data according to the command header I found in the previous step, then I use the data to set the corresponding variables"

	waitForData := waitForData - 1.
	storedInputData at: waitForData + 1 put: inputByte.

	(waitForData = 0
			and: [executeMultiByteCommand ~= 0])
		ifTrue: [executeMultiByteCommand caseOf: {
				[FirmataConstants analogMessage] -> [self setAnalogInput: multiByteChannel value: (storedInputData first bitShift: 7)
						+ storedInputData second].
				[FirmataConstants digitalMessage] -> [
						self setDigitalInputs: multiByteChannel data: (storedInputData first bitShift: 7)
						+ storedInputData second].
				[FirmataConstants reportVersion] -> [self majorVersion: storedInputData first minorVersion: storedInputData second]}
				 otherwise: [].
			executeMultiByteCommand := 0]! !

!Firmata methodsFor: 'receiving' stamp: 'Richo 9/8/2009 10:31'!
parseSysex: inputByte 
	inputByte = FirmataConstants endSysex
		ifTrue: [parsingSysex := false]
		ifFalse: [storedInputData at: sysexBytesRead + 1 put: inputByte.
			sysexBytesRead := sysexBytesRead + 1
				]! !

!Firmata methodsFor: 'digital pins' stamp: 'Richo 1/12/2010 13:09'!
pin: pin mode: mode
"Sends an individual Pin Mode message"
self port nextPutAll: (ByteArray with: FirmataConstants setPinMode with: pin with: mode).! !

!Firmata methodsFor: 'sound' stamp: 'Richo 1/18/2013 18:21'!
playTone: frequency duration: duration pin: pin
	self halt! !

!Firmata methodsFor: 'accessing' stamp: 'Richo 1/6/2012 13:29'!
port
	self controlConnection.
	^port! !

!Firmata methodsFor: 'printing' stamp: 'Richo 8/24/2009 15:14'!
printOn: aStream
super printOn: aStream.
aStream nextPutAll: '(',self hash asString,')'! !

!Firmata methodsFor: 'receiving' stamp: 'Richo 9/14/2009 16:26'!
processInput
	| data |
	data := port readByteArray.

	data
		do: [:inputByte | 

parsingSysex
				ifTrue: [self parseSysex: inputByte]
				ifFalse: [(waitForData > 0
							and: [inputByte < 128])
						ifTrue: [self parseData: inputByte]
						ifFalse: [self parseCommandHeader: inputByte]]]! !

!Firmata methodsFor: 'other commands' stamp: 'Richo 8/18/2009 12:42'!
queryFirmware
"Send a Firmware Name and Version query"
self startSysex.
self port  nextPutAll: (ByteArray with: FirmataConstants reportFirmware ).! !

!Firmata methodsFor: 'other commands' stamp: 'Richo 1/31/2011 10:06'!
queryVersion
"Send a version report query"
self port nextPutAll: (ByteArray with: FirmataConstants reportVersion ).! !

!Firmata methodsFor: 'AFMotorShield' stamp: ''!
runMotor: numMotor command: cmd 
	self
		runMotor: numMotor
		command: cmd
		speed: 100! !

!Firmata methodsFor: 'AFMotorShield' stamp: ''!
runMotor: numMotor command: cmd speed: speed 
	| cmdCode t5 |
	cmd = #Stop
		ifTrue: [cmdCode := 3].
	cmd = #Backward
		ifTrue: [cmdCode := 2].
	cmd = #Forward
		ifTrue: [cmdCode := 1].
	t5 := ByteArray new: 6.
	t5 at: 1 put: FirmataConstants startSysex;
		 at: 2 put: FirmataConstants shiftData;
		 at: 3 put: numMotor;
		 at: 4 put: cmdCode;
		 at: 5 put: (speed * 2.5) rounded;
		 at: 6 put: FirmataConstants endSysex.
	self port nextPutAll: t5! !

!Firmata methodsFor: 'keep alive' stamp: 'Richo 12/11/2012 14:09'!
sendKeepAlive
	(self port nextPutAll: {0} asByteArray) < 0
		ifTrue: [self disconnect]! !

!Firmata methodsFor: 'servos' stamp: 'Richo 8/26/2009 16:14'!
servoConfig: pin minPulse: minPulse maxPulse: maxPulse angle: angle
	"Default values: 
	minPulse 544 (LSB 32 MSB 4)
	maxPulse 2400 (LSB 96 MSB 18
	"
	| b |
	b := ByteArray new: 10.
	b at: 1 put: FirmataConstants startSysex;
		 at: 2 put: FirmataConstants servoConfig;
		 at: 3 put: pin;
		 at: 4 put:  (minPulse rounded bitAnd: 127);
		 at: 5 put: ((minPulse rounded bitShift: -7)
				bitAnd: 127);
		 at: 6 put: (maxPulse rounded bitAnd: 127);
		 at: 7 put: ((maxPulse rounded bitShift: -7)
				bitAnd: 127);
		 at: 8 put: (angle rounded bitAnd: 127);
		 at: 9 put: ((angle rounded bitShift: -7)
				bitAnd: 127);
		 at: 10 put: FirmataConstants endSysex.
	self port nextPutAll: b! !

!Firmata methodsFor: 'servos' stamp: 'Richo 8/26/2009 17:00'!
servoOnPin: pinNumber angle: degrees
self analogWrite: pinNumber value: degrees! !

!Firmata methodsFor: 'analog pins' stamp: 'Richo 8/18/2009 13:01'!
setAnalogInput: pin value: value
"Stores analog value data in array"
	analogInputData at: pin + 1 put: value! !

!Firmata methodsFor: 'digital pins' stamp: 'Richo 8/18/2009 13:01'!
setDigitalInputs: portNumber data: portData 
"Puts but masked On/Off data for a whole digital port into array"
	digitalInputData at: portNumber + 1 put: portData! !

!Firmata methodsFor: 'stepping' stamp: 'Richo 3/9/2010 11:36'!
startSteppingProcess
	steppingProcess notNil
		ifTrue: [^ self].
	steppingProcess := [[self step.
			(Delay forMilliseconds: self stepTime) wait] repeat]
				forkNamed: self asString ! !

!Firmata methodsFor: 'other commands' stamp: 'Richo 8/18/2009 12:41'!
startSysex
"Sends a Start Sysex message"
self port  nextPutAll: (ByteArray with: FirmataConstants startSysex  ).! !

!Firmata methodsFor: 'stepping' stamp: 'Richo 9/9/2009 18:00'!
step
	[self processInput]
		on: Error
		do: [:err | 
			Transcript show: self asString , ': ' , err description;
				 cr. self initializeVariables.
			"self resetConnection"]! !

!Firmata methodsFor: 'stepping' stamp: 'Richo 8/31/2009 14:47'!
stepTime
	^ 1! !

!Firmata methodsFor: 'stepping' stamp: 'Richo 8/23/2011 12:57'!
stopSteppingProcess
	steppingProcess isNil
		ifTrue: [^ self].
	steppingProcess terminate. 
	steppingProcess := nil! !

!Firmata methodsFor: 'pin modes' stamp: 'Richo 3/9/2010 10:47'!
valueForInputMode
^FirmataConstants pinModeInput! !

!Firmata methodsFor: 'pin modes' stamp: 'Richo 3/9/2010 10:47'!
valueForOutputMode
^FirmataConstants pinModeOutput ! !

!Firmata methodsFor: 'pin modes' stamp: 'Richo 4/30/2010 15:44'!
valueForPwmMode
^FirmataConstants pinModePwm! !

!Firmata methodsFor: 'pin modes' stamp: 'Richo 3/9/2010 10:47'!
valueForServoMode
^FirmataConstants pinModeServo ! !

!Firmata methodsFor: 'accessing' stamp: 'Richo 3/30/2010 13:11'!
version
"self queryVersion."
^(majorVersion asString , '.', minorVersion asString) asNumber! !

Object subclass: #ArduinoType
	instanceVariableNames: 'fCpu friendlyNames mcu uploadRate firmataRate programmer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core'!

ArduinoType class
	instanceVariableNames: 'types'!

ArduinoType class
	instanceVariableNames: 'types'!

!ArduinoType class methodsFor: 'accessing types' stamp: 'Richo 1/31/2011 09:55'!
allFriendlyNames
	^ types inject: #() into: [:subTotal :next | subTotal, next friendlyNames]! !

!ArduinoType class methodsFor: 'accessing types' stamp: 'Richo 1/31/2011 09:46'!
at: aSymbol
	^ types at: aSymbol! !

!ArduinoType class methodsFor: 'accessing' stamp: 'Richo 11/23/2012 12:21'!
default
	^ self at: #ArduinoUNO! !

!ArduinoType class methodsFor: 'instance creation' stamp: 'Richo 9/15/2011 17:48'!
friendlyNames: anArray fCpu: anInteger mcu: aString uploadRate: uploadRate firmataRate: firmataRate
	^ self basicNew initializeWithFriendlyNames: anArray fCpu: anInteger mcu: aString uploadRate: uploadRate firmataRate: firmataRate avrdudeProgrammer: 'stk500'! !

!ArduinoType class methodsFor: 'instance creation' stamp: 'Richo 9/15/2011 17:48'!
friendlyNames: anArray fCpu: anInteger mcu: aString uploadRate: uploadRate firmataRate: firmataRate avrdudeProgrammer: programmerString
	^ self basicNew initializeWithFriendlyNames: anArray fCpu: anInteger mcu: aString uploadRate: uploadRate firmataRate: firmataRate avrdudeProgrammer: programmerString! !

!ArduinoType class methodsFor: 'class initialization' stamp: 'Richo 9/15/2011 17:50'!
initialize
"
self initialize
"
types := Dictionary new.

types at: #ArduinoDiecimila
	 put: (ArduinoType 
		friendlyNames: #('diecimila w/ atmega168' 'duemilanove w/ atmega168' 'nano w/ atmega168')
		fCpu: 16000000
		mcu: 'atmega168'
		uploadRate: 19200
		firmataRate: 57600).

types at: #ArduinoLilyPad
	 put: (ArduinoType 
		friendlyNames: #('lilypad w/ atmega168')
		fCpu: 8000000
		mcu: 'atmega168'
		uploadRate: 19200
		firmataRate: 57600).

types at: #ArduinoMega
	 put: (ArduinoType 
		friendlyNames: #('mega')
		fCpu: 16000000
		mcu: 'atmega1280'
		uploadRate: 57600
		firmataRate: 57600).

types at: #ArduinoBT
	 put: (ArduinoType 
		friendlyNames: #('bluetooth')
		fCpu: 16000000
		mcu: 'atmega168'
		uploadRate: 19200
		firmataRate: 57600).

types at: #ArduinoATmega8
	 put: (ArduinoType 
		friendlyNames: #('ng or older w/ atmega8')
		fCpu: 16000000
		mcu: 'atmega8'
		uploadRate: 19200
		firmataRate: 115200).

types at: #ArduinoMini
	 put: (ArduinoType 
		friendlyNames: #('mini')
		fCpu: 16000000
		mcu: 'atmega168'
		uploadRate: 19200
		firmataRate: 57600).

types at: #ArduinoATmega168
	 put: (ArduinoType 
		friendlyNames: #('ng or older w/ atmega168')
		fCpu: 16000000
		mcu: 'atmega168'
		uploadRate: 19200
		firmataRate: 57600).

types at: #ArduinoLilyPad328
	 put: (ArduinoType 
		friendlyNames: #('lilypad w/ atmega328')
		fCpu: 8000000
		mcu: 'atmega328p'
		uploadRate: 57600
		firmataRate: 57600).

types at: #ArduinoATmega328
	 put: (ArduinoType 
		friendlyNames: #('duemilanove w/ atmega328' 'nano w/ atmega328')
		fCpu: 16000000
		mcu: 'atmega328p'
		uploadRate: 57600
		firmataRate: 57600).

types at: #ArduinoPro
	 put: (ArduinoType 
		friendlyNames: #('pro w/ atmega168' 'pro mini w/ atmega168')
		fCpu: 8000000
		mcu: 'atmega168'
		uploadRate: 19200
		firmataRate: 57600).

types at: #ArduinoPro328
	 put: (ArduinoType 
		friendlyNames: #('pro w/ atmega328' 'pro mini w/ atmega328')
		fCpu: 8000000
		mcu: 'atmega328p'
		uploadRate: 57600
		firmataRate: 57600).

types	at: #ArduinoMega2560
	put: (ArduinoType
		friendlyNames: #('mega 2560' )
		fCpu: 16000000
		mcu: 'atmega2560'
		uploadRate: 115200
		firmataRate: 57600
		avrdudeProgrammer: 'stk500v2').

types	at: #ArduinoUNO
	put: (ArduinoType
		friendlyNames: #('uno' )
		fCpu: 16000000
		mcu: 'atmega328p'
		uploadRate: 115200
		firmataRate: 57600)! !

!ArduinoType class methodsFor: 'accessing types' stamp: 'Richo 11/23/2012 12:32'!
types
	^ types! !

!ArduinoType class methodsFor: 'accessing types' stamp: 'Richo 4/6/2011 10:43'!
withFriendlyName: aSymbol 
"Return an ArduinoType with the friendly name given as a parameter"

	^ types detect: [:each | each friendlyNames includes: aSymbol] ifNone: [self error: 'ArduinoType not found' translated]! !

!ArduinoType methodsFor: 'accessing' stamp: 'Richo 9/15/2011 17:47'!
avrdudeProgrammer
	^programmer! !

!ArduinoType methodsFor: 'accessing' stamp: 'Richo 1/31/2011 09:31'!
fCpu
	"Answer the value of fCpu"

	^ fCpu! !

!ArduinoType methodsFor: 'accessing' stamp: 'Richo 1/31/2011 09:31'!
firmataRate
	"Answer the value of firmataRate"

	^ firmataRate! !

!ArduinoType methodsFor: 'accessing' stamp: 'Richo 1/31/2011 09:31'!
friendlyNames
	"Answer the value of friendlyNames"

	^ friendlyNames! !

!ArduinoType methodsFor: 'accessing' stamp: 'Richo 11/23/2012 12:32'!
id
	^ self class types keyAtValue: self! !

!ArduinoType methodsFor: 'private' stamp: 'Richo 9/15/2011 17:48'!
initializeWithFriendlyNames: anArray fCpu: anInteger mcu: aString uploadRate: uploadRate1 firmataRate: firmataRate1 avrdudeProgrammer: programmerString
	self initialize.
	friendlyNames := anArray.
	fCpu := anInteger.
	mcu := aString.
	uploadRate := uploadRate1.
	firmataRate := firmataRate1.
	programmer := programmerString! !

!ArduinoType methodsFor: 'accessing' stamp: 'Richo 1/31/2011 09:31'!
mcu
	"Answer the value of mcu"

	^ mcu! !

!ArduinoType methodsFor: 'accessing' stamp: 'Richo 1/31/2011 09:31'!
uploadRate
	"Answer the value of uploadRate"

	^ uploadRate! !

Object subclass: #AttachableDevice
	instanceVariableNames: 'pins isConfigured'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!AttachableDevice commentStamp: '<historical>' prior: 0!
Abstract class. I represent a device that can be attached to a collection of Arduino's pins. My protocol is very simple:

#attach: aPin
Attaches the device to a single pin and sets the pins configuration accordingly.

#isAttached
Checks if the device is attached to a pin.

#detach
Simply detachs the device from the pins and deletes the configuration. It raises an error if the device is not attached.

Subclasses should override #configure and #deleteConfiguration with the specific steps to configure the corresponding device.!

AttachableDevice subclass: #AFMotor
	instanceVariableNames: 'mode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-AFMotorShield'!

!AFMotor methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:30'!
configure
	pins first mode: AFMotorMode new
! !

!AFMotor methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:30'!
deleteConfiguration
	pins first setOutput
! !

!AFMotor methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 19:03'!
direction
	self controlAttaching; reconfigureIfNecessary.
	^ pins first direction
! !

!AFMotor methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 19:03'!
direction: d
	self controlAttaching; reconfigureIfNecessary.
	pins first direction: d
! !

!AFMotor methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 19:03'!
speed
	self controlAttaching; reconfigureIfNecessary.
	^ pins first speed! !

!AFMotor methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 19:03'!
speed: s
	self controlAttaching; reconfigureIfNecessary.
	pins first
		speed: (s min: 100 max: 0)! !

!AFMotor methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 19:03'!
state
	self controlAttaching; reconfigureIfNecessary.
	^ pins first state
! !

!AFMotor methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 19:03'!
state: s
	self controlAttaching; reconfigureIfNecessary.
	pins first state: s
! !

!AttachableDevice methodsFor: 'attaching' stamp: 'Richo 6/1/2010 14:20'!
attach: aPin
	self setPins: (Array with: aPin)! !

!AttachableDevice methodsFor: 'attaching' stamp: 'Richo 6/1/2010 15:08'!
attachAnalogPins: anArray on: anArduino 
"
	self attachAnalogPins: #(1 2 3 4) on: Arduino new.
"
	self setPins: (anArray collect: [:pinNumber| anArduino analogPin: pinNumber]).! !

!AttachableDevice methodsFor: 'attaching' stamp: 'Richo 6/1/2010 15:08'!
attachDigitalPins: anArray on: anArduino 
"
	self attachDigitalPins: #(1 2 3 4) on: Arduino new.
"
	self setPins: (anArray collect: [:pinNumber| anArduino digitalPin: pinNumber]).! !

!AttachableDevice methodsFor: 'configuring' stamp: 'Richo 9/6/2011 11:54'!
configure
	"This method is responsible of configuring the pin for using the corresponding device. Most of the times the configuration is simply setting the correct pin mode"
	pins do: [:each | each configure: self]! !

!AttachableDevice methodsFor: 'configuring' stamp: 'Richo 9/6/2011 11:56'!
configureAnalog: anAnalogPin
	self subclassResponsibility ! !

!AttachableDevice methodsFor: 'configuring' stamp: 'Richo 9/6/2011 11:56'!
configureDigital: aDigitalPin
	self subclassResponsibility ! !

!AttachableDevice methodsFor: 'attaching' stamp: 'Richo 9/8/2009 12:52'!
controlAttaching
self isAttached
		ifFalse: [^ self unattachedError].! !

!AttachableDevice methodsFor: 'configuring' stamp: 'Richo 9/6/2011 11:55'!
deleteConfiguration
	"This method simply resets the pin's configuration to its default state"
	pins do: [:each | each deleteConfiguration: self]! !

!AttachableDevice methodsFor: 'configuring' stamp: 'Richo 9/6/2011 12:01'!
deleteConfigurationAnalog: anAnalogPin
	self subclassResponsibility ! !

!AttachableDevice methodsFor: 'configuring' stamp: 'Richo 9/6/2011 12:02'!
deleteConfigurationDigital: aDigitalPin
	self subclassResponsibility ! !

!AttachableDevice methodsFor: 'attaching' stamp: 'Richo 6/1/2010 14:21'!
detach
	self controlAttaching.
	[self deleteConfiguration] on: Error do: ["The port may be disconnected"].
	pins := nil! !

!AttachableDevice methodsFor: 'attaching' stamp: 'Richo 8/15/2011 10:14'!
isAttached
	^pins notNil! !

!AttachableDevice methodsFor: 'attaching' stamp: 'Richo 8/15/2011 10:13'!
isAttachedTo: anArduino 
	^ self isAttached and: [anArduino pins includesAllOf: pins]! !

!AttachableDevice methodsFor: 'attaching' stamp: 'Richo 8/15/2011 16:20'!
isAttachedToArduino: anArduino 
	^ self isAttached and: [anArduino pins includesAllOf: pins]! !

!AttachableDevice methodsFor: 'attaching' stamp: 'Richo 8/15/2011 16:20'!
isAttachedToPin: aPin
	^ self isAttached and: [pins includes: aPin]! !

!AttachableDevice methodsFor: 'configuring' stamp: 'Richo 6/1/2010 14:21'!
reconfigureIfNecessary
	"This method must be called every time we try to communicate with the
	device. This way we are sure that the device is properly configured"
	isConfigured
		ifFalse: [self setPins: pins]! !

!AttachableDevice methodsFor: 'private' stamp: 'Richo 6/1/2010 14:21'!
setPins: anArrayOfPins
"When attaching a device I try to configure it, but if it fails for some reason I simply set isConfigured to false, so that we remember to configure whenever possible"
	pins := anArrayOfPins.
	[self configure.
	isConfigured := true]
		on: Error
		do: [isConfigured := false]! !

!AttachableDevice methodsFor: 'errors' stamp: 'Richo 4/6/2011 10:40'!
unattachedError
^self error: 'Unattached device: ' translated, self asString.! !

AttachableDevice subclass: #Buzzer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!Buzzer methodsFor: 'configuring' stamp: 'Richo 9/6/2011 11:57'!
configureDigital: aPin
	aPin mode: PwmMode new! !

!Buzzer methodsFor: 'configuring' stamp: 'Richo 9/6/2011 12:01'!
deleteConfigurationDigital: aPin
	aPin setOutput! !

!Buzzer methodsFor: 'accessing' stamp: 'Richo 1/18/2013 18:26'!
playTone: frequency duration: duration
	| pin |
	self controlAttaching; reconfigureIfNecessary.
	pin := pins first.
	pin protocol
		playTone: frequency duration: duration pin: pin number! !

!Buzzer methodsFor: 'accessing' stamp: 'Richo 9/7/2011 11:09'!
value
	self controlAttaching; reconfigureIfNecessary.
	^ pins first value! !

!Buzzer methodsFor: 'accessing' stamp: 'Richo 9/7/2011 11:09'!
value: aNumber 
	"aNumber can be 0 - 255"
	self controlAttaching; reconfigureIfNecessary.
	pins first value: (aNumber min: 255 max: 0)! !

AttachableDevice subclass: #DCMotor
	instanceVariableNames: 'speed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!DCMotor methodsFor: 'accessing' stamp: 'Richo 6/27/2011 17:53'!
brake
	self controlAttaching; reconfigureIfNecessary.
	self d0Pin value: 1.
	self d1Pin value: 1.
	self enablePin value: 255! !

!DCMotor methodsFor: 'configuring' stamp: 'Richo 9/6/2011 12:00'!
configureDigital: aPin
	| index mode |
	index := pins indexOf: aPin ifAbsent: [^ self error].
	mode := index caseOf: {
		[1] -> [OutputMode].
		[2] -> [OutputMode].
		[3] -> [PwmMode]}.
	aPin mode: mode new;
		value: 0! !

!DCMotor methodsFor: 'private' stamp: 'Richo 6/27/2011 15:26'!
d0Pin
	^ pins first! !

!DCMotor methodsFor: 'private' stamp: 'Richo 6/27/2011 15:26'!
d1Pin
	^ pins second! !

!DCMotor methodsFor: 'configuring' stamp: 'Richo 9/6/2011 12:01'!
deleteConfigurationDigital: aPin
	aPin setOutput! !

!DCMotor methodsFor: 'private' stamp: 'Richo 6/27/2011 15:26'!
enablePin
	^ pins third! !

!DCMotor methodsFor: 'initialize-release' stamp: 'Richo 6/27/2011 15:33'!
initialize
	super initialize.
	speed := 0.! !

!DCMotor methodsFor: 'accessing' stamp: 'Richo 6/27/2011 15:33'!
speed
	^ speed! !

!DCMotor methodsFor: 'accessing' stamp: 'Richo 6/27/2011 17:52'!
speed: aNumber
	self controlAttaching; reconfigureIfNecessary.
	speed := aNumber min: 100 max: -100.
	speed < 0
		ifTrue: [self d0Pin value: 0.
			self d1Pin value: 1]
		ifFalse: [self d0Pin value: 1.
			self d1Pin value: 0].
	self enablePin value: (speed abs / 100 * 255)
	! !

AttachableDevice subclass: #EButton
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!EButton commentStamp: 'Richo 9/15/2009 12:07' prior: 0!
I represent a button (digital input device).!

!EButton methodsFor: 'configuring' stamp: 'Richo 10/25/2011 16:42'!
configureAnalog: aPin
	aPin setInput activate! !

!EButton methodsFor: 'configuring' stamp: 'Richo 10/25/2011 15:47'!
configureDigital: aPin
	aPin setInput! !

!EButton methodsFor: 'configuring' stamp: 'Richo 9/6/2011 12:16'!
deleteConfigurationAnalog: aPin
	aPin deactivate ! !

!EButton methodsFor: 'configuring' stamp: 'Richo 9/6/2011 12:02'!
deleteConfigurationDigital: aPin
aPin setOutput! !

!EButton methodsFor: 'accessing' stamp: 'Richo 1/7/2013 11:25'!
value
	self controlAttaching ; reconfigureIfNecessary .
	^ pins first isDigital
		ifTrue: [pins first value]
		ifFalse: [(pins first value / 1023) rounded]
! !

AttachableDevice subclass: #Keypad
	instanceVariableNames: 'buttons steppingProcess'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!Keypad methodsFor: 'accessing' stamp: 'Richo 6/1/2010 15:13'!
at: rowNumber at: columnNumber
	^self atRow: rowNumber column: columnNumber! !

!Keypad methodsFor: 'accessing' stamp: 'Richo 6/1/2010 15:13'!
atRow: rowNumber column: columnNumber
	self controlAttaching; reconfigureIfNecessary.
	^buttons at: rowNumber at: columnNumber! !

!Keypad methodsFor: 'private' stamp: 'Richo 5/31/2010 18:18'!
columnPins
^pins first: 3! !

!Keypad methodsFor: 'configuring' stamp: 'Richo 9/6/2011 12:07'!
configureDigital: aPin
	(self rowPins includes: aPin) ifTrue: [aPin setOutput].
	(self columnPins includes: aPin) ifTrue: [aPin setInput]! !

!Keypad methodsFor: 'configuring' stamp: 'Richo 9/6/2011 12:08'!
deleteConfigurationDigital: aPin
	aPin setOutput.
	self stopSteppingProcess ! !

!Keypad methodsFor: 'initialize-release' stamp: 'Richo 6/8/2010 15:35'!
finalize
	self stopSteppingProcess.! !

!Keypad methodsFor: 'initialize-release' stamp: 'Richo 6/8/2010 17:28'!
initialize
	super initialize.
	self finalizationRegistry add: self.
	buttons := Matrix rows: 3 columns: 3 element: false.! !

!Keypad methodsFor: 'private' stamp: 'Richo 5/31/2010 18:00'!
rowPins
^pins last: 3! !

!Keypad methodsFor: 'attaching' stamp: 'Richo 5/31/2010 18:04'!
rowPins: rowPinNumbers columnsPins: columnPinNumbers on: anArduino
"
	self rowPins: #(7 6 5) columnPins: #(2 3 4) on: Arduino new.
"
self attachDigitalPins: columnPinNumbers , rowPinNumbers on: anArduino! !

!Keypad methodsFor: 'private' stamp: 'Richo 9/6/2011 12:07'!
setPins: anArrayOfPins
	super setPins: anArrayOfPins.
	isConfigured ifTrue: [self startSteppingProcess]! !

!Keypad methodsFor: 'stepping' stamp: 'Richo 6/8/2010 17:25'!
startSteppingProcess
	steppingProcess notNil
		ifTrue: [^ self].
	steppingProcess := [[self step] repeat] forkNamed: self asString ! !

!Keypad methodsFor: 'stepping' stamp: 'Richo 5/31/2010 18:27'!
step
1 to: 3 do: [:r |
	[self rowPins first value: (r = 1) asBit.
	self rowPins second value: (r = 2) asBit.
	self rowPins third value: (r = 3) asBit.
	50 milliSeconds asDelay wait.
	1 to: 3 do: [:c |
		buttons at: r at: c put: ((self columnPins at: c) value = 1).
	]] on: Error do: [200 milliSeconds asDelay wait]]! !

!Keypad methodsFor: 'stepping' stamp: 'Richo 5/31/2010 17:59'!
stepTime
	^ 1! !

!Keypad methodsFor: 'stepping' stamp: 'Richo 5/31/2010 17:59'!
stopSteppingProcess
	steppingProcess isNil
		ifTrue: [^ self].
	steppingProcess terminate. 
	steppingProcess := nil.! !

AttachableDevice subclass: #LightEmittingDiode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!LightEmittingDiode commentStamp: 'Richo 9/15/2009 12:08' prior: 0!
I represent a led (digital output device).!

!LightEmittingDiode methodsFor: 'configuring' stamp: 'Richo 10/25/2011 16:43'!
configureAnalog: aPin
	aPin deactivate.
	aPin setOutput! !

!LightEmittingDiode methodsFor: 'configuring' stamp: 'Richo 10/25/2011 15:47'!
configureDigital: aPin
	aPin setOutput! !

!LightEmittingDiode methodsFor: 'configuring' stamp: 'Richo 9/6/2011 13:22'!
deleteConfigurationAnalog: aPin
	aPin setInput! !

!LightEmittingDiode methodsFor: 'configuring' stamp: 'Richo 9/6/2011 11:57'!
deleteConfigurationDigital: aPin
	aPin setOutput! !

!LightEmittingDiode methodsFor: 'accessing' stamp: 'Richo 6/1/2010 14:23'!
value
	self controlAttaching;  reconfigureIfNecessary.
	^ pins first value! !

!LightEmittingDiode methodsFor: 'accessing' stamp: 'Richo 6/1/2010 14:23'!
value: aNumber 
	self controlAttaching;reconfigureIfNecessary.
	pins first value: aNumber! !

AttachableDevice subclass: #Optocoupler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!Optocoupler methodsFor: 'configuring' stamp: 'Richo 10/25/2011 16:43'!
configureAnalog: aPin
	aPin setInput activate! !

!Optocoupler methodsFor: 'configuring' stamp: 'Richo 9/6/2011 12:54'!
deleteConfigurationAnalog: aPin
	aPin deactivate! !

!Optocoupler methodsFor: 'accessing' stamp: 'Richo 9/6/2011 12:54'!
value
	self controlAttaching; reconfigureIfNecessary.
	^pins first value! !

AttachableDevice subclass: #Photoresistor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!Photoresistor commentStamp: 'Richo 9/15/2009 12:08' prior: 0!
I represent an LDR (analog input device)!

!Photoresistor methodsFor: 'configuring' stamp: 'Richo 10/25/2011 16:42'!
configureAnalog: aPin
	aPin setInput activate! !

!Photoresistor methodsFor: 'configuring' stamp: 'Richo 9/6/2011 12:10'!
deleteConfigurationAnalog: aPin
	aPin deactivate! !

!Photoresistor methodsFor: 'accessing' stamp: 'Richo 6/1/2010 14:24'!
value
	self controlAttaching; reconfigureIfNecessary.
	^pins first value! !

AttachableDevice subclass: #Potentiometer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!Potentiometer commentStamp: 'Richo 9/15/2009 12:09' prior: 0!
I represent a Potentiometer (analog input device).!

!Potentiometer methodsFor: 'configuring' stamp: 'Richo 10/25/2011 16:43'!
configureAnalog: aPin
	aPin setInput activate! !

!Potentiometer methodsFor: 'configuring' stamp: 'Richo 9/6/2011 12:10'!
deleteConfigurationAnalog: aPin
	aPin deactivate! !

!Potentiometer methodsFor: 'accessing' stamp: 'Richo 6/1/2010 14:24'!
value
	self controlAttaching; reconfigureIfNecessary.
	^ pins first value! !

AttachableDevice subclass: #PwmLightEmittingDiode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!PwmLightEmittingDiode methodsFor: 'configuring' stamp: 'Richo 9/6/2011 12:10'!
configureDigital: aPin
	aPin mode: PwmMode new! !

!PwmLightEmittingDiode methodsFor: 'configuring' stamp: 'Richo 9/6/2011 12:10'!
deleteConfigurationDigital: aPin
	aPin setOutput! !

!PwmLightEmittingDiode methodsFor: 'accessing' stamp: 'Richo 6/1/2010 14:24'!
value
	self controlAttaching;  reconfigureIfNecessary.
	^ pins first value! !

!PwmLightEmittingDiode methodsFor: 'accessing' stamp: 'Richo 6/1/2010 15:06'!
value: aNumber 
	"aNumber can be 0 - 255"
	self controlAttaching;reconfigureIfNecessary.
	pins first value: (aNumber min: 255 max: 0)! !

AttachableDevice subclass: #Relay
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!Relay methodsFor: 'configuring' stamp: 'Richo 12/27/2011 16:10'!
configureAnalog: aPin
	aPin deactivate.
	aPin setOutput! !

!Relay methodsFor: 'configuring' stamp: 'Richo 12/27/2011 16:10'!
configureDigital: aPin
	aPin setOutput! !

!Relay methodsFor: 'configuring' stamp: 'Richo 12/27/2011 16:16'!
deleteConfigurationAnalog: aPin
	aPin value: 0.
	aPin setInput! !

!Relay methodsFor: 'configuring' stamp: 'Richo 12/27/2011 16:16'!
deleteConfigurationDigital: aPin
	aPin value: 0.
	aPin setOutput! !

!Relay methodsFor: 'accessing' stamp: 'Richo 12/27/2011 16:11'!
value
	self controlAttaching;  reconfigureIfNecessary.
	^ pins first value! !

!Relay methodsFor: 'accessing' stamp: 'Richo 12/27/2011 16:11'!
value: aNumber 
	self controlAttaching;reconfigureIfNecessary.
	pins first value: aNumber! !

AttachableDevice subclass: #Servo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!Servo commentStamp: 'Richo 9/15/2009 12:11' prior: 0!
I represent a servo motor.!

!Servo methodsFor: 'accessing' stamp: 'Richo 6/1/2010 15:21'!
angle
	self controlAttaching; reconfigureIfNecessary.
	^pins first value! !

!Servo methodsFor: 'accessing' stamp: 'Richo 10/24/2011 15:13'!
angle: aSmallInteger 
	self controlAttaching; reconfigureIfNecessary.
	pins first value = aSmallInteger ifTrue: [^ self].
	pins first value: (aSmallInteger min: 180 max: 0)! !

!Servo methodsFor: 'configuring' stamp: 'Richo 9/6/2011 12:10'!
configureDigital: aPin
	aPin mode: ServoMode new.! !

!Servo methodsFor: 'configuring' stamp: 'Richo 9/6/2011 12:11'!
deleteConfigurationDigital: aPin
	aPin setOutput! !

!Servo methodsFor: 'initialize-release' stamp: 'Richo 4/30/2010 16:57'!
initialize
super initialize.! !

AttachableDevice subclass: #StepperMotor
	instanceVariableNames: 'steppingProcess stepTime stepMode isGoingForward currentStep'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!StepperMotor methodsFor: 'configuring' stamp: 'Richo 9/6/2011 12:11'!
configureDigital: aPin
	aPin setOutput; value: 0
! !

!StepperMotor methodsFor: 'configuring' stamp: 'Richo 9/6/2011 12:11'!
deleteConfigurationDigital: aPin
	"Nothing to do here"
! !

!StepperMotor methodsFor: 'stepping' stamp: 'Richo 12/13/2012 15:49'!
doNextStep
	[self perform: stepMode.
	currentStep := currentStep + (isGoingForward ifTrue: [1] ifFalse: [-1])] 
		on: Error
		do: [200 milliSeconds asDelay wait]! !

!StepperMotor methodsFor: 'initialize-release' stamp: 'Richo 12/13/2012 15:48'!
finalize
	self stopSteppingProcess
! !

!StepperMotor methodsFor: 'actions' stamp: 'Richo 6/8/2010 17:05'!
goBackwards
	isGoingForward := false! !

!StepperMotor methodsFor: 'actions' stamp: 'Richo 6/8/2010 17:05'!
goForward
	isGoingForward := true! !

!StepperMotor methodsFor: 'step modes' stamp: 'Richo 12/13/2012 15:50'!
halfStepMode
	| values |
	values := (self halfStepModeSteps at: currentStep \\ 8 + 1).
	1 to: 4 do: [:i | (pins at: i) value: (values at: i)]! !

!StepperMotor methodsFor: 'step modes' stamp: 'Richo 6/8/2010 16:33'!
halfStepModeSteps
^#(
	#(1 0 0 0)
	#(1 1 0 0)
	#(0 1 0 0)
	#(0 1 1 0)
	#(0 0 1 0)
	#(0 0 1 1)
	#(0 0 0 1)
	#(1 0 0 1)
)! !

!StepperMotor methodsFor: 'initialize-release' stamp: 'Richo 12/13/2012 16:09'!
initialize
	super initialize.
	self finalizationRegistry add: self.
	stepTime := 100.
	isGoingForward := true.
	currentStep := 1.
	self setNormalMode! !

!StepperMotor methodsFor: 'accessing' stamp: 'Richo 12/13/2012 15:49'!
isGoingForward
	^ isGoingForward ! !

!StepperMotor methodsFor: 'accessing' stamp: 'Richo 12/13/2012 16:09'!
isRunning
	^ steppingProcess notNil! !

!StepperMotor methodsFor: 'accessing' stamp: 'Richo 12/13/2012 15:51'!
isUsingHalfStepMode
	^ stepMode = #waveDriveMode! !

!StepperMotor methodsFor: 'accessing' stamp: 'Richo 12/13/2012 15:51'!
isUsingNormalMode
	^ stepMode = #normalMode! !

!StepperMotor methodsFor: 'accessing' stamp: 'Richo 12/13/2012 15:51'!
isUsingWaveDriveMode
	^ stepMode = #waveDriveMode! !

!StepperMotor methodsFor: 'step modes' stamp: 'Richo 12/13/2012 15:50'!
normalMode
	| values |
	values := (self normalModeSteps at: currentStep \\ 4 + 1).
	1 to: 4 do: [:i | (pins at: i) value: (values at: i)]! !

!StepperMotor methodsFor: 'step modes' stamp: 'Richo 6/8/2010 16:32'!
normalModeSteps
^#(
	#(1 1 0 0)
	#(0 1 1 0)
	#(0 0 1 1)
	#(1 0 0 1)
)! !

!StepperMotor methodsFor: 'actions' stamp: 'Richo 6/8/2010 17:05'!
reverse
	isGoingForward := isGoingForward not! !

!StepperMotor methodsFor: 'actions' stamp: 'Richo 12/13/2012 15:50'!
setHalfStepMode
	stepMode := #halfStepMode! !

!StepperMotor methodsFor: 'actions' stamp: 'Richo 12/13/2012 15:50'!
setNormalMode
	stepMode := #normalMode! !

!StepperMotor methodsFor: 'actions' stamp: 'Richo 12/13/2012 15:50'!
setWaveDriveMode
	stepMode := #waveDriveMode! !

!StepperMotor methodsFor: 'actions' stamp: 'Richo 12/13/2012 16:09'!
start
	self controlAttaching;
		reconfigureIfNecessary;
		startSteppingProcess! !

!StepperMotor methodsFor: 'stepping' stamp: 'Richo 6/7/2010 18:20'!
startSteppingProcess
	steppingProcess notNil
		ifTrue: [^ self].
	steppingProcess := [[self step.
			(Delay forMilliseconds: self stepTime) wait] repeat]
				forkNamed: self asString ! !

!StepperMotor methodsFor: 'stepping' stamp: 'Richo 12/13/2012 16:09'!
step
	self doNextStep! !

!StepperMotor methodsFor: 'stepping' stamp: 'Richo 12/13/2012 15:49'!
stepTime
	^ stepTime! !

!StepperMotor methodsFor: 'actions' stamp: 'Richo 4/6/2011 11:06'!
stepTime: aNumber
	aNumber < 1 ifTrue: [^self error: 'Step time can''t be less than 1' translated].
	stepTime := aNumber.! !

!StepperMotor methodsFor: 'actions' stamp: 'Richo 12/13/2012 16:09'!
stop
	self controlAttaching;
		reconfigureIfNecessary;
		stopSteppingProcess! !

!StepperMotor methodsFor: 'stepping' stamp: 'Richo 6/7/2010 18:21'!
stopSteppingProcess
	steppingProcess isNil
		ifTrue: [^ self].
	steppingProcess terminate. 
	steppingProcess := nil.! !

!StepperMotor methodsFor: 'step modes' stamp: 'Richo 12/13/2012 15:50'!
waveDriveMode
	| values |
	values := (self waveDriveModeSteps at: currentStep \\ 4 + 1).
	1 to: 4 do: [:i | (pins at: i) value: (values at: i)]! !

!StepperMotor methodsFor: 'step modes' stamp: 'Richo 6/8/2010 16:32'!
waveDriveModeSteps
^#(
	#(1 0 0 0)
	#(0 1 0 0)
	#(0 0 1 0)
	#(0 0 0 1)
)! !

AttachableDevice subclass: #Thermistor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!Thermistor methodsFor: 'configuring' stamp: 'Richo 10/25/2011 16:43'!
configureAnalog: aPin
	aPin setInput activate! !

!Thermistor methodsFor: 'configuring' stamp: 'Richo 9/6/2011 12:11'!
deleteConfigurationAnalog: aPin
	aPin deactivate! !

!Thermistor methodsFor: 'accessing' stamp: 'Richo 6/1/2010 14:25'!
value
	self controlAttaching; reconfigureIfNecessary.
	^pins first value! !

AttachableDevice subclass: #TiltSwitch
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!TiltSwitch methodsFor: 'configuring' stamp: 'Richo 10/25/2011 15:47'!
configureDigital: aPin
	aPin setInput! !

!TiltSwitch methodsFor: 'configuring' stamp: 'Richo 9/6/2011 12:12'!
deleteConfigurationDigital: aPin
	aPin setOutput! !

!TiltSwitch methodsFor: 'accessing' stamp: 'Richo 6/1/2010 14:25'!
value
self controlAttaching ; reconfigureIfNecessary .
^pins first value! !

Object subclass: #FirmataConstants
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Firmata'!

!FirmataConstants commentStamp: 'Richo 9/15/2009 12:00' prior: 0!
See class side. I collect a bunch of constants for the Firmata protocol.!

!FirmataConstants class methodsFor: 'message commands' stamp: 'Richo 8/18/2009 10:35'!
analogMessage
"0xE0 send data for an analog pin (or PWM)"
^16rE0! !

!FirmataConstants class methodsFor: 'message commands' stamp: 'Richo 8/18/2009 10:35'!
digitalMessage
"0x90 send data for digital pin"
^16r90! !

!FirmataConstants class methodsFor: 'message commands' stamp: 'Richo 8/18/2009 10:39'!
endSysex
"0xF7 end a MIDI Sysex message"
^16rF7! !

!FirmataConstants class methodsFor: 'version numbers' stamp: 'Richo 8/18/2009 10:33'!
firmataMajorVersion
"for non-compatible changes"
^2! !

!FirmataConstants class methodsFor: 'version numbers' stamp: 'Richo 8/18/2009 10:33'!
firmataMinorVersion
"for backward compatible changes"
^0! !

!FirmataConstants class methodsFor: 'extended command set using Sysex' stamp: 'Richo 8/18/2009 10:41'!
firmataString
"0x71 a string message with 14-bits per character"
^16r71! !

!FirmataConstants class methodsFor: 'others' stamp: 'Richo 8/18/2009 10:34'!
maxDataBytes
"Maximum number of data bytes in non-Sysex messages"
^32 ! !

!FirmataConstants class methodsFor: 'pin modes' stamp: 'Richo 9/7/2009 16:16'!
pinModeAnalog
"analog pin in analogInput mode"
^2! !

!FirmataConstants class methodsFor: 'pin modes' stamp: 'Richo 9/7/2009 16:16'!
pinModeInput
^0! !

!FirmataConstants class methodsFor: 'pin modes' stamp: 'Richo 9/7/2009 16:16'!
pinModeOutput
^1! !

!FirmataConstants class methodsFor: 'pin modes' stamp: 'Richo 9/7/2009 16:17'!
pinModePwm
"digital pin in PWM output mode"
^3! !

!FirmataConstants class methodsFor: 'pin modes' stamp: 'Richo 9/7/2009 16:17'!
pinModeServo 
"digital pin in Servo output mode"
^4! !

!FirmataConstants class methodsFor: 'pin modes' stamp: 'Richo 9/11/2009 14:37'!
pinModeUnavailable
	^ -1! !

!FirmataConstants class methodsFor: 'message commands' stamp: 'Richo 8/18/2009 10:36'!
reportAnalog
"0xC0 enable analog input by pin number"
^16rC0! !

!FirmataConstants class methodsFor: 'message commands' stamp: 'Richo 8/18/2009 10:36'!
reportDigital
"0xD0 enable digital input by port pair"
^16rD0! !

!FirmataConstants class methodsFor: 'extended command set using Sysex' stamp: 'Richo 8/18/2009 10:41'!
reportFirmware
"0x79 report name and version of the firmware"
^16r79! !

!FirmataConstants class methodsFor: 'message commands' stamp: 'Richo 8/18/2009 10:38'!
reportVersion
"0xF9 report protocol version"
^16rF9! !

!FirmataConstants class methodsFor: 'extended command set using Sysex' stamp: 'Richo 8/18/2009 10:40'!
servoConfig
"0x70 set maximum angle, minPulse, maxPulse, frequency"
^16r70! !

!FirmataConstants class methodsFor: 'message commands' stamp: 'Richo 8/18/2009 10:37'!
setPinMode
"0xF4 set a pin to INPUT/OUTPUT/ANALOG/PWM/SERVO - 0/1/2/3/4"
^16rF4! !

!FirmataConstants class methodsFor: 'message commands' stamp: ''!
shiftData
	^ 117! !

!FirmataConstants class methodsFor: 'message commands' stamp: 'Richo 8/18/2009 10:38'!
startSysex
"0xF0 start a MIDI Sysex message"
^16rF0! !

!FirmataConstants class methodsFor: 'extended command set using Sysex' stamp: 'Richo 8/18/2009 10:41'!
sysexNonRealtime
"0x7E MIDI reserved for non-realtime messages"
^16r7E! !

!FirmataConstants class methodsFor: 'extended command set using Sysex' stamp: 'Richo 8/18/2009 10:42'!
sysexRealtime
"0x7F MIDI reserved for realtime messages"
^16r7F! !

!FirmataConstants class methodsFor: 'message commands' stamp: 'Richo 8/18/2009 10:38'!
systemReset
"0xFF reset from MIDI"
^16rFF! !

!FirmataConstants class methodsFor: 'version numbers' stamp: 'Richo 8/18/2009 10:33'!
versionBlickPin
"digital pin to blick version on"
^13! !

Object subclass: #PinMode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-PinMode'!

PinMode class
	instanceVariableNames: 'uniqueInstance'!

!PinMode commentStamp: 'Richo 9/15/2009 12:15' prior: 0!
Abstract class. I represent the mode of an Arduino digital pin. Tipically, I would just need input/output mode, other subclasses were made for practical purposes (i.e. ButtonMode, LedMode).!

PinMode class
	instanceVariableNames: 'uniqueInstance'!

PinMode subclass: #InputMode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-PinMode'!

!InputMode commentStamp: 'Richo 9/15/2009 12:13' prior: 0!
I represent the input mode of an Arduino digital pin.!

!InputMode methodsFor: 'testing' stamp: 'Richo 10/24/2011 15:08'!
isInput
	^ true! !

!InputMode methodsFor: 'accessing' stamp: 'Richo 9/7/2009 16:35'!
nameSymbol
	^#input! !

!InputMode methodsFor: 'refreshing' stamp: 'Richo 4/30/2010 16:08'!
refreshModeOn: pinNumber using: anArduinoProtocol
	anArduinoProtocol digitalPin: pinNumber mode: anArduinoProtocol valueForInputMode.
! !

!InputMode methodsFor: 'refreshing' stamp: 'Richo 4/6/2011 10:42'!
refreshValue: value on: pinNumber using: anArduinoProtocol
^self error: 'Input mode cannot set value' translated! !

PinMode subclass: #OutputMode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-PinMode'!

!OutputMode commentStamp: 'Richo 9/15/2009 12:13' prior: 0!
I represent the output mode of an Arduino digital pin.!

OutputMode subclass: #AFMotorMode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-AFMotorShield'!

!AFMotorMode methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:25'!
nameSymbol
	^ #dcmotor
! !

!AFMotorMode methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:26'!
refreshValue: value on: pinNumber using: anArduinoProtocol
	anArduinoProtocol runMotor: pinNumber command: value
! !

!OutputMode methodsFor: 'testing' stamp: 'Richo 9/7/2009 16:29'!
isOutput
^true! !

!OutputMode methodsFor: 'accessing' stamp: 'Richo 9/7/2009 16:34'!
nameSymbol
	^#output! !

!OutputMode methodsFor: 'refresing' stamp: 'Richo 4/30/2010 16:08'!
refreshModeOn: pinNumber using: anArduinoProtocol
	anArduinoProtocol digitalPin: pinNumber mode: anArduinoProtocol valueForOutputMode.
! !

!OutputMode methodsFor: 'refresing' stamp: 'Richo 4/30/2010 16:01'!
refreshValue: value on: pinNumber using: anArduinoProtocol
	anArduinoProtocol digitalWrite: pinNumber value: value! !

OutputMode subclass: #PwmMode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-PinMode'!

!PwmMode commentStamp: '<historical>' prior: 0!
I represent the pwm mode of an Arduino digital pin.!

!PwmMode methodsFor: 'testing' stamp: 'Richo 10/24/2011 15:08'!
isPwm
	^ true! !

!PwmMode methodsFor: 'accessing' stamp: 'Richo 4/30/2010 15:43'!
nameSymbol
	^#pwm! !

!PwmMode methodsFor: 'refreshing' stamp: 'Richo 4/30/2010 16:08'!
refreshModeOn: pinNumber using: anArduinoProtocol
	anArduinoProtocol digitalPin: pinNumber mode: anArduinoProtocol valueForPwmMode.
! !

!PwmMode methodsFor: 'refreshing' stamp: 'Richo 4/30/2010 16:03'!
refreshValue: value on: pinNumber using: anArduinoProtocol
	anArduinoProtocol analogWrite: pinNumber value: value! !

PwmMode subclass: #ServoMode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-PinMode'!

!ServoMode commentStamp: 'Richo 9/15/2009 12:13' prior: 0!
I represent the servo mode of an Arduino digital pin.!

!ServoMode methodsFor: 'testing' stamp: 'Richo 12/29/2011 18:53'!
isServo
	^ true! !

!ServoMode methodsFor: 'accessing' stamp: 'Richo 9/7/2009 16:35'!
nameSymbol
	^#servo! !

!ServoMode methodsFor: 'refreshing' stamp: 'Richo 4/30/2010 16:13'!
refreshModeOn: pinNumber using: anArduinoProtocol
	anArduinoProtocol attachServoToPin: pinNumber;
						digitalPin: pinNumber mode: anArduinoProtocol valueForServoMode.
! !

!ServoMode methodsFor: 'refreshing' stamp: 'Richo 4/30/2010 16:43'!
refreshValue: value on: pinNumber using: anArduinoProtocol
	anArduinoProtocol servoOnPin: pinNumber angle: value! !

!PinMode class methodsFor: 'as yet unclassified' stamp: 'Richo 9/7/2009 16:23'!
new
^uniqueInstance ifNil: [uniqueInstance := self basicNew]! !

!PinMode methodsFor: 'testing' stamp: 'Richo 10/24/2011 15:08'!
isInput
^false! !

!PinMode methodsFor: 'testing' stamp: 'Richo 9/7/2009 16:29'!
isOutput
^false! !

!PinMode methodsFor: 'testing' stamp: 'Richo 10/24/2011 15:08'!
isPwm
^false! !

!PinMode methodsFor: 'testing' stamp: 'Richo 12/29/2011 18:52'!
isServo
^false! !

!PinMode methodsFor: 'accessing' stamp: 'Richo 9/7/2009 16:33'!
nameSymbol
^#undefined! !

!PinMode methodsFor: 'setting pins' stamp: 'Richo 4/30/2010 16:07'!
refreshModeOn: pinNumber using: anArduinoProtocol
	self subclassResponsibility! !

!PinMode methodsFor: 'setting pins' stamp: 'Richo 4/30/2010 16:00'!
refreshValue: value on: pinNumber using: anArduinoProtocol
	self subclassResponsibility! !

Object subclass: #Queue
	instanceVariableNames: 'list'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Firmware'!

!Queue methodsFor: 'accessing' stamp: 'Richo 8/29/2011 15:13'!
flush
	list notEmpty ifTrue: [list := OrderedCollection new]! !

!Queue methodsFor: 'initialize-release' stamp: 'Richo 8/29/2011 15:10'!
initialize
	super initialize.
	list := OrderedCollection new.! !

!Queue methodsFor: 'accessing' stamp: 'Richo 8/29/2011 15:10'!
pop
	^ list removeFirst! !

!Queue methodsFor: 'printing' stamp: 'Richo 8/29/2011 14:17'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	list do: [:each | each printOn: aStream] separatedBy: [aStream nextPutAll: ', '].
	aStream nextPut: $)! !

!Queue methodsFor: 'accessing' stamp: 'Richo 8/29/2011 15:09'!
push: obj
	list addLast: obj! !

Error subclass: #FirmwareNotInstalled
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Firmware'!

AttachableDeviceMorph subclass: #AFMotorMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-AFMotorShield'!

!AFMotorMorph class methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:49'!
additionsToViewerCategories
	^ #(
	(#dcmotor (
	#(#slot state 'State of the DC motor: stopped or running' #MotorState #readWrite #Player #getState #Player #setState:)
	#(#slot motorDirection 'Rotation direction DC motor: forward or backward' #MotorDirection #readWrite #Player #getMotorDirection #Player #setMotorDirection:)
	#(#slot speed 'Speed of the DC motor, from 0% to 100%' #Number #readWrite #Player #getSpeed #Player #setSpeed:)

)) )! !

!AFMotorMorph class methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 17:43'!
defaultNameStemForInstances
	^'DC Motor' translatedNoop! !

!AFMotorMorph class methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 17:43'!
device
	^ AFMotor! !

!AFMotorMorph class methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:28'!
initialize
	"
	self initialize
	"
	Vocabulary addStandardVocabulary: (SymbolListType new vocabularyName: #MotorState;
			 symbols: #(#Running #Stopped )).
	Vocabulary addStandardVocabulary: (SymbolListType new vocabularyName: #MotorDirection;
			 symbols: #(#Forward #Backward ))
! !

!AFMotorMorph class methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 17:43'!
initializeForm
	^ form := PEUtilities formFromEncodedPNG: 'iVBORw0KGgoAAAANSUhEUgAAAFIAAACWCAYAAABNVd85AAAABHNCSVQICAgIfAhkiAAAAAlw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'! !

!AFMotorMorph class methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 17:44'!
newConnector
	^ ArduinoWireMorph new
		 lineColor: Color yellow;
		borderColor: Color gray;
		lineWidth: 5;
		 yourself! !

!AFMotorMorph class methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 17:44'!
newConnectorFromMorph: m1 toMorph: m2
^ (ArduinoWireMorph fromMorph: m1 toMorph: m2)
		 lineColor: Color yellow;
		borderColor: Color gray;
		lineWidth: 5;
		 yourself! !

!AFMotorMorph methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:40'!
attachmentPointsSpecs
	^ {
		"{#center. #color. #borderColor}"
		{self center + (0@71). Color yellow. Color gray}
	}! !

!AFMotorMorph methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 19:05'!
direction
	^ [device direction] on: Error do: [#error]
! !

!AFMotorMorph methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 19:06'!
direction: d 
	[device direction: d] on: Error do: ["Nothing"]
! !

!AFMotorMorph methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 17:45'!
speed
	^ [device speed] on: Error do: [-1]! !

!AFMotorMorph methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 19:06'!
speed: aNumber
	^ [device speed: aNumber] on: Error do: ["Nothing"]! !

!AFMotorMorph methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 19:06'!
state
	^ [device state] on: Error do: [#error]
! !

!AFMotorMorph methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 19:06'!
state: s 
	[device state: s] on: Error do: ["Nothing"]
! !

ArduinoMorph subclass: #AFMotorshieldMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-AFMotorShield'!

!AFMotorshieldMorph class methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:06'!
descriptionForPartsBin
	^ self
		partName: 'Arduino with Adafruit Motorshield' translatedNoop
		categories: (Project current isPEProject
						ifTrue: [{'Physical Etoys' translatedNoop}]
						ifFalse: [#()])
		documentation: 'An arduino with the Adafruit MotorShield' translatedNoop
! !

!AFMotorshieldMorph class methodsFor: 'as yet unclassified' stamp: ''!
initializeForm
	^ form := PEUtilities formFromEncodedPNG: 'iVBORw0KGgoAAAANSUhEUgAAAVwAAAENCAYAAABDxN1+AAAABHNCSVQICAgIfAhkiAAAAAlw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'! !

!AFMotorshieldMorph methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 19:02'!
initialize
	super initialize.
	self extent: self class form extent.
	arduino := AFMotorShield new.
	boardType := self class lastBoardType.
	self initializePortName;
		initializePins;
		initializeSpecialPins ! !

!AFMotorshieldMorph methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:38'!
initializePins
	| t1 |
	t1 := 83 @ 125.
	0
		to: 5
		do: [:t2 | 
			self
				addMorph: ((ArduinoAnalogPinMorph
						on: (arduino analogPin: t2))
						center: self center + t1).
			t1 := t1 x + 13 @ t1 y].
	self
		addMorph: ((ArduinoDigitalPinMorph
				on: (arduino digitalPin: 13))
				center: self center + (-33 @ -124)).
	self
		addMorph: ((ArduinoDigitalPinMorph
				on: (arduino digitalPin: 10))
				center: self center + (-137 @ -122)).
	self
		addMorph: ((ArduinoDigitalPinMorph
				on: (arduino digitalPin: 9))
				center: self center + (-137 @ -109)).
	self
		addMorph: ((ArduinoDigitalPinMorph
				on: (arduino digitalPin: 2))
				center: self center + (122 @ -124))! !

!AFMotorshieldMorph methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:38'!
initializeSpecialPins
	self
		addMorph: ((AFMotorPinMorph
				on: (arduino motorPin: 1))
				center: self center + (-180 @ 0)).
	self
		addMorph: ((AFMotorPinMorph
				on: (arduino motorPin: 2))
				center: self center + (-180 @ 75)).
	self
		addMorph: ((AFMotorPinMorph
				on: (arduino motorPin: 4))
				center: self center + (180 @ -32)).
	self
		addMorph: ((AFMotorPinMorph
				on: (arduino motorPin: 3))
				center: self center + (180 @ 40))! !

!AFMotorshieldMorph methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:09'!
isMotorPin1Active
	^ (arduino motorPin: 1) isActive
! !

!AFMotorshieldMorph methodsFor: 'as yet unclassified' stamp: 'Richo 4/8/2013 18:09'!
motorPin1
	^ [(arduino motorPin: 1) value]
		on: Error
		do: [-1]
! !
ArduinoType initialize!
AFMotorMorph initialize!
